package rootgui.controller;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import rootgui.model.AbstractModel;
import rootgui.model.Geometry;
import rootgui.model.IDataProvider;
import rootgui.model.IHistDataProvider;
import rootgui.model.ITabHistDataProvider;
import rootgui.model.SimulationRun;
import rootgui.model.TabPropertyChangeEvent;
import rootgui.model.objects.Cell;
import rootgui.model.objects.SimulationModel;
import rootgui.model.objects.Species;
import rootgui.model.recordstore.Record;
import rootgui.model.recordstore.RecordStoreActionListener;
import rootgui.view.AbstractViewFrame;
import rootgui.view.IConsoleOutput;
import rootgui.view.SystemOutputAdapter;
import visualiser.Figure;
import visualiser.IVisualiser;

/**
 * 
 * @author tghetiu, based on the SUN MVC example
 */
public abstract class AbstractController implements PropertyChangeListener,
		ITabSimulationControl, ITabHistDataProvider, RecordStoreActionListener {

	/*
	 * 
	 */
	private ArrayList<AbstractViewFrame> registeredViews;

	// private ArrayList<AbstractModel> registeredModels;
	/*
	 * 
	 */
	protected AbstractModel model;

	/*
	 * 
	 */
//	protected IVisualiser vis;

	/*
	 * 
	 */
	protected IConsoleOutput consO = new SystemOutputAdapter();
	
	/*
	 * 
	 */
	protected RunnableBlockingQueueList rbql;

	public AbstractController() {
		registeredViews = new ArrayList<AbstractViewFrame>();
		// registeredModels = new ArrayList<AbstractModel>();
	}

	public void setModel(AbstractModel model) {
		this.model = model;
		model.addPropertyChangeListener(this);
	}

	public void unsetModel() {
		// registeredModels.remove(model);
		model.removePropertyChangeListener(this);
	}

	public void addView(AbstractViewFrame view) {
		registeredViews.add(view);
		rbql.addListener(view);
	}

	public void setConsoleOutput(IConsoleOutput ico) {
		consO = ico;
	}

	public void removeView(AbstractViewFrame view) {
		registeredViews.remove(view);
		rbql.removeListener(view);
	}
	
//	public void setVisualiser(IVisualiser vis) {
//		this.vis = vis;
//	}
//
//	public IVisualiser getVisualiser() {
//		return vis;
//	}	
	
	// Use this to observe property changes from registered models
	// and propagate them on to all the views.

	public void propertyChange(PropertyChangeEvent evt) {

		for (AbstractViewFrame view : registeredViews) {
			view.modelPropertyChange((TabPropertyChangeEvent)evt);
		}
	}

	/**
	 * This is a convenience method that subclasses can call upon to fire
	 * property changes back to the models. This method uses reflection to
	 * inspect each of the model classes to determine whether it is the owner of
	 * the property in question. If it isn't, a NoSuchMethodException is thrown,
	 * which the method ignores.
	 * 
	 * @param propertyName
	 *            = The name of the property.
	 * @param newValue
	 *            = An object that represents the new value of the property.
	 */
	protected void setModelProperty(String propertyName, int simID, Object newValue) {

		// for (AbstractModel model : registeredModels) {
		try {

			Method method = model.getClass().getMethod("set" + propertyName,
					new Class[] { int.class, newValue.getClass() }

			);
			method.invoke(model, simID, newValue);

		} catch (Exception ex) {
			ex.printStackTrace();
		}
		// }
	}
	
	public abstract void createSimulationRun(final int simID);
	
	public abstract void removeSimulationRun(final int simID);
	
	public abstract void removeAllSimulationRuns();
	
	public abstract int getSimulationRunCount();

	public abstract void newSimulation(final int simID);

	public abstract void setIteration(int simID, int iteration);
	
	public abstract void resetSimulation(final int simID);
}
