package rootgui.controller;

import rootgui.model.SimulationRun;
import rootgui.model.objects.SimulationModel;
import rootgui.model.objects.Species;
import rootgui.model.recordstore.Record;

public interface ITabSimulationControl {
	/*
	 * Perform one iteration
	 */
	public void step(int simID);
	
	public void stepNonBlocking(int simID);

	// public void stepAll();

	public void cancel(int simID);

	public void setSimulationInitialised(int simID);
	/*
	 * Specifies the tissue file name
	 */
	public void setTissueFileName(int simID, String fileName);

	/*
	 * 
	 */
	public String getTissueFileName(int simID);

	/*
	 * Initialises the tissue
	 */
	public void initTissue(int simID);

	/*
	 * Sets a model parameter
	 */
	public void setModelParameter(int simID, String modelName,
			String parameter, String value);

	/*
	 * Specifies the maximum number of snapshots (images) the simulation should
	 * output
	 */
	public void setMaxIterations(int simID, int iterations);

	/*
	 * 
	 */
	public void setTimestep(int simID, double timestep);

	/*
	 * Returns true if there is an active connection to the simulation server
	 */
	public boolean isConnected(int simID);

	/*
	 * Disconnects the simulation server
	 */
	public void disconnectSimulation(int simID);

	public void setIteration(int simID, int iteration);

	public void updateData(int simID, Species targetSpecies);

	public void updateData(int simID);

	/*
	 * Adds a SimulationModel to the simulation
	 */
	public void loadSimulationModel(int simID, SimulationModel sm);
	
	public void loadSimulationModel(final int simID, final String modelName);

	public void unloadSimulationModel(int simID, String modelName);

	public void setCellCarrier(int simID, String[] groups, boolean[] onOff);
	
	public void setSimulationRun(int simID, SimulationRun simRun);

	public void setGrowth(int simID, boolean onOff);
	
//	public String getSpeciesPublicName(int simID, Species species);
}
