package rootgui.model;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import rootgui.controller.VRootController;
import rootgui.model.objects.SimulationModel;
import rootgui.model.objects.Species;
import rootgui.model.recordstore.IRecordStore;
import rootgui.model.recordstore.Record;
import rootgui.model.recordstore.RecordStoreActionListener;

/**
 * 
 * @author tghetiu, based on Sun's MVC example
 */
public abstract class AbstractModel implements ITabHistDataProvider
{

	/*
	 * List of SimulationRun objects
	 */
	protected Map<Integer, SimulationRun> simulationRuns;

	public abstract void clearRecordStore(int simID);

	protected PropertyChangeSupport propertyChangeSupport;

	public AbstractModel() {
		propertyChangeSupport = new PropertyChangeSupport(this);
	}

	/*
     * 
     */
	public void addPropertyChangeListener(PropertyChangeListener listener) {
		propertyChangeSupport.addPropertyChangeListener(listener);
	}

	/*
     * 
     */
	public void removePropertyChangeListener(PropertyChangeListener listener) {
		propertyChangeSupport.removePropertyChangeListener(listener);
	}

	/*
     * 
     */
	protected void firePropertyChange(int simID, String propertyName,
			Object oldValue, Object newValue) {
		TabPropertyChangeEvent pce = new TabPropertyChangeEvent(simID, this,
				propertyName, oldValue, newValue);
		propertyChangeSupport.firePropertyChange(pce);

	}

	/*
	 * Specifies that a new iteration (with a given Record of data) is available
	 */
	public abstract int setIncIteration(int simID, Record r);

	/*
	 * Sets the first record of a SimulationRun's RecordStore
	 */
	public abstract int setInitialRecord(int simID, Record r);

	/*
	 * Specifies the maximum number of iterations (time snapshots) to be
	 * obtained
	 */
	public abstract void setMaxIterations(int simID, Integer maxIterations);

	/* SPECIFIC */

	/*
	 * Returns the geometry for the current iteration
	 */
	// public abstract Geometry getGeometry();

	/*
	 * 
	 */
	// public abstract void setGeometry(Geometry geometry);

	/*
	 * Returns the geometry for a given iteration
	 */
	public abstract Geometry getGeometry(int simID, int iteration);

	/*
	 * Loads the geometry for the current iteration, from a specified file
	 */
	public void readGeometryFromFile(int simID, String path) throws Exception {
		readGeometryFromFile(simID, path, simulationRuns.get(simID)
				.getIteration());
	}

	/*
	 * Loads geometry for a given iteration
	 */
	public abstract void readGeometryFromFile(int simID, String path,
			int iteration) throws Exception;

	/*
	 * Loads a species dataset, for the current iteration
	 */
	public void readSpeciesFromFile(int simID, Species species, String csvFile)
			throws Exception {
		readSpeciesFromFile(simID, species, csvFile,
				simulationRuns.get(simID).getIteration());
	}

	/*
	 * Loads a concentration file
	 */
	public abstract void readSpeciesFromFile(int simID, Species species,
			String csvFile, int iteration) throws Exception;

	/*
	 * Clears all model state
	 */
	public abstract void clear(int simID);

	/*
	 * Sets the name of the tissue file
	 */
	public abstract void setTissueFileName(int simID, String fileName);

	public abstract String getTissueFileName(int simID);

	/*
	 * Specifies the duration of a time step
	 */
	public abstract void setTimestep(int simID, Double timestep);

	/*
	 * Adds a SimulationModel object to the list of available simulation models.
	 */
	public abstract void setAvailableSimulationModel(int simID,
			SimulationModel sm);

	/*
	 * 
	 */
	public abstract void clearAvailableSimulationModels(int simID);

	/*
	 * 
	 */
	public abstract void setModelParameter(int simID, String modelName,
			String parameter, String value);

	/*
	 * Specifies that a SimulationModel has been loaded in the simulation
	 */
	public abstract void setModelLoaded(int simID, SimulationModel sm)
			throws Exception;

	/*
	 * Specifies that a SimulationModel has been unloaded from the simulation
	 */
	public abstract void setModelUnloaded(int simID, String modelName)
			throws Exception;

	/*
	 * 
	 */
	public abstract int removeIterations(int simID, int startingIteration);

	public abstract boolean isExecutionAllowed(int simID);

	public abstract boolean isTissueFileInitialised(int simID);

	public abstract void setTissueFileInitialised(int simID, Record r);

	public abstract void createSimulationRun(final int simID, RecordStoreActionListener rsal);
	
	public abstract void removeSimulationRun(int simID);
	
	/*
	 * Sets a given SimulationRun over an existing simID
	 */
	public abstract void setSimulationRun(int simID, SimulationRun sr);

	/*
	 * Creates a new SimulationRun, with a given simID
	 */
	public abstract void newSimulationRun(int simID, RecordStoreActionListener rsal);

	// public abstract void setCurrentSimulationRun(SimulationRun simRun);
	//
	// public abstract SimulationRun getCurrentSimulationRun();
}
