package rootgui.model.objects;

import java.io.Serializable;
import java.util.Comparator;

/*
 * Types of species that exist in the simulation
 */
public class Species implements Serializable, Comparable<Species> {
	/**
	 * 
	 */
	private static final long serialVersionUID = -7826934725101620137L;
	
	private String name;
	private String publicName;
	Type type;

//	private boolean selected;
	
	public Species(String speciesName, String publicName, Type type) {
		this.name = speciesName;
		this.publicName = publicName;
		this.type = type;
	}

	// public void setSelected (boolean selected){
	// this.selected = selected;
	// }
	//
	// public boolean isSelected(){
	// return selected;
	// }
	public String getName() {
		return name;
	}
	
	public String getPublicName(){
		return publicName;
	}

	public Type getType() {
		return type;
	}

	public boolean getIsItalics(){
		return false;
	}
	
	public enum Type{
		CELL("cell"), CELL_WALL("(cell,wall)"), WALL("wall"); 

		private String text;

		Type(String text) {
			this.text = text;
		}

		public static Type fromString(String text) {
			if (text != null) {
				for (Type t : Type.values())
					if (text.equalsIgnoreCase(t.text))
						return t;
			}
			return null;
		}
		
		public static Comparator<Type> getComparator() {
			
			return new Comparator<Species.Type>() {
				
				@Override
				public int compare(Type o1, Type o2) {
					switch(o1){
					case CELL:
						switch(o2){
						case CELL:
							return 0;
						case CELL_WALL:
							return 1;
						case WALL:
							return -1;
						}
					case WALL:
						switch(o2){
						case CELL:
							return -1;
						case CELL_WALL:
							return 1;
						case WALL:
							return 0;
						}
					case CELL_WALL:
						switch(o2){
						case CELL:
							return -1;
						case CELL_WALL:
							return 0;
						case WALL:
							return 1;
						}
					}
					return 0;
				}
			};
		}
		
		
	}
	
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof Species) {
			Species o=(Species) obj;
			return name.equalsIgnoreCase(o.name) && type.equals(o.type);
		} else {
			return false;
		}
	}
	
	@Override
	public int hashCode() {
		return name.hashCode()+type.hashCode();
	}
	
	

	@Override
	public String toString() {
		return name;
	}

	@Override
	public int compareTo(Species o) {
		int r= type.compareTo(o.type);
//		int r= Type.getComparator().compare(type, o.type);

		if (r!=0) {
			return r;
		} else {
			return name.compareToIgnoreCase(o.name);
		}
	}
}
