package rootgui.model.parser;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.Hashtable;
import java.util.LinkedList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import rootgui.model.objects.*;
import rootgui.model.Geometry;

import rootgui.model.exceptions.ImproperlyStructuredCellWallsException;

public class CellsetGeometryParser implements ICellGeometryParser {
	Document doc;

	public CellsetGeometryParser() {
	}

	@Override
	public Geometry getGeometryFromFile(String xmlFile) {
		try {
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder dBuilder = null;

			dBuilder = dbFactory.newDocumentBuilder();
			doc = dBuilder.parse(new File(xmlFile));
			doc.getDocumentElement().normalize();

			return parseXML(doc);
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	public Geometry getGeometryFromString(String xmlString) {
		DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder dBuilder = null;

		try {
			dBuilder = dbFactory.newDocumentBuilder();
			doc = dBuilder.parse(new InputSource(new StringReader(xmlString)));		
			doc.getDocumentElement().normalize();
			return parseXML(doc);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		return null;
	}

	/*
	 * This method assumes the model is empty
	 */
	private Geometry parseXML(Document doc) throws XPathExpressionException {
		Geometry geometry = new Geometry();

		
		
		XPathFactory factory = XPathFactory.newInstance();
		XPath xpath = factory.newXPath();
		XPathExpression expr = xpath.compile("/cellsetdata/walls");
		Boolean isWallNotInlined = (Boolean) expr.evaluate(doc,
				XPathConstants.BOOLEAN);
		if (isWallNotInlined) {
			XPathExpression wallsQuery = xpath
					.compile("/cellsetdata/walls/wall");
			NodeList wallsList = (NodeList) wallsQuery.evaluate(doc,
					XPathConstants.NODESET);
			Hashtable<Integer, Wall> walltable = new Hashtable<Integer, Wall>();
			for (int i = 0; i < wallsList.getLength(); i++) {
				Node wall = wallsList.item(i);
				assert wall instanceof Element;
				if (wall instanceof Element) {
					Element ew = (Element) wall;
					Integer id = Integer.parseInt(ew.getAttribute("id"));
					walltable.put(id, buildWall(id, ew));
				}

			}
			NodeList cellss = doc.getElementsByTagName("cells");
			assert cellss.getLength() == 1;
			NodeList cells = cellss.item(0).getChildNodes();
			for (int i = 0; i < cells.getLength(); i++) {
				Node cell = cells.item(i);
				assert cell instanceof Element;
				if (cell instanceof Element) {
					geometry.addCell(buildCellWithoutWallsInlined(
							(Element) cell, walltable));
				}
			}
			assert geometry.getCellCount() == getCellCount();
		} else {
			NodeList cellss = doc.getElementsByTagName("cells");
			assert cellss.getLength() == 1;
			NodeList cells = cellss.item(0).getChildNodes();
			for (int i = 0; i < cells.getLength(); i++) {
				Node cell = cells.item(i);
				assert cell instanceof Element;
				if (cell instanceof Element) {
					geometry.addCell(buildCellWithWallsInlined((Element) cell));
				}
			}
			assert geometry.getCellCount() == getCellCount();
		}
		
		
		

		updateCellGroupName(geometry, doc);
		updateWallGroupName(geometry, doc);
		
		return geometry;
	}
	
	
	private void updateCellGroupName(Geometry geo, Document doc) throws XPathExpressionException {
		XPathFactory factory = XPathFactory.newInstance();
		XPath xpath = factory.newXPath();
		XPathExpression cellgroupsExpr = xpath.compile("/cellsetdata/groups/cellgroups/group");
		NodeList cellgroupEntries = (NodeList) cellgroupsExpr.evaluate(doc,
				XPathConstants.NODESET);
		// System.out.println(cellgroupEntries.getLength());
		for (int i = 0; i < cellgroupEntries.getLength(); i++) {
			Node cellgroupEntry = cellgroupEntries.item(i);
			assert cellgroupEntry instanceof Element;
			if (cellgroupEntry instanceof Element) {
				Element ew = (Element) cellgroupEntry;
				Integer id = Integer.parseInt(ew.getAttribute("id"));
				String name=ew.getAttribute("name");
				geo.addCellGroupName(id, name);
			}
		}
	}
	
	private void updateWallGroupName(Geometry geo, Document doc) throws XPathExpressionException {
		XPathFactory factory = XPathFactory.newInstance();
		XPath xpath = factory.newXPath();
		XPathExpression wallgroupsExpr = xpath.compile("/cellsetdata/groups/wallgroups/group");
		NodeList wallgroupEntries = (NodeList) wallgroupsExpr.evaluate(doc,
				XPathConstants.NODESET);
		for (int i = 0; i < wallgroupEntries.getLength(); i++) {
			Node cellgroupEntry = wallgroupEntries.item(i);
			assert cellgroupEntry instanceof Element;
			if (cellgroupEntry instanceof Element) {
				Element ew = (Element) cellgroupEntry;
				Integer id = Integer.parseInt(ew.getAttribute("id"));
				String name=ew.getAttribute("name");
				geo.addWallGroupName(id, name);
			}
		}
	}

	// System.out.println(model.getCells().size());
	// for (Cell c : model.getCells()) {
	// System.out.println(c);
	// }

	private Cell buildCellWithWallsInlined(Element cellElement) {
		int id = Integer.parseInt(cellElement.getAttribute("id"));
		int group = Integer.parseInt(cellElement.getAttribute("group"));
		boolean truncated = Boolean.parseBoolean(cellElement
				.getAttribute("truncated"));
		LinkedList<Wall> walls = new LinkedList<Wall>();
		NodeList wallNodes = ((Element) cellElement.getElementsByTagName(
				"walls").item(0)).getElementsByTagName("wall");
		for (int i = 0; i < wallNodes.getLength(); i++) {
			Node node = wallNodes.item(i);
			if (node instanceof Element) {
				Element wallElement = (Element) node;
				Integer Wid = Integer.parseInt(wallElement.getAttribute("id"));
				walls.add(buildWall(Wid,wallElement));
			}
		}
		Cell cell = new Cell(id, group, walls);
		cell.setTruncated(truncated);
		return cell;
	}

	private Cell buildCellWithoutWallsInlined(Element cellElement,
			Hashtable<Integer, Wall> walltable) {
		int id = Integer.parseInt(cellElement.getAttribute("id"));
		int group = Integer.parseInt(cellElement.getAttribute("group"));
		boolean truncated = Boolean.parseBoolean(cellElement
				.getAttribute("truncated"));
		LinkedList<Wall> walls = new LinkedList<Wall>();
		LinkedList<Wall> connectedWalls = new LinkedList<Wall>(); // walls that
																	// form
																	// a closed
																	// circuit
		NodeList wallNodes = ((Element) cellElement.getElementsByTagName(
				"walls").item(0)).getElementsByTagName("wall");

		for (int i = 0; i < wallNodes.getLength(); i++) {
			Node node = wallNodes.item(i);
			if (node instanceof Element) {
				Element wallElement = (Element) node;
				Integer wallId = Integer.parseInt(wallElement
						.getAttribute("id"));
				Wall w = walltable.get(wallId);
				w.setCellID(id);
				walls.add(w);
			}
		}

		connectedWalls.add(new Wall(walls.remove(0)));

		int maxIteration = walls.size();
		for (int i = 0; i < maxIteration; i++) {
			for (Wall w : walls) {
				if (connectedWalls.getLast().getTail().equals(w.getHead())) {
					walls.remove(w);
					connectedWalls.add(new Wall(w));
					break;
				} else if (connectedWalls.getLast().getTail()
						.equals(w.getTail())) {
					walls.remove(w);
					connectedWalls.add(w.getReverse());
					break;				
			}	else if (w.getHead().equals(w.getTail())) {
				//remove any walls that have no length, problem comes from rounding to get integers from a scaled geometry
				walls.remove(w);
//				System.out.println("CellsetGeometryParser: Removing null-length wall");
				break;
			}
			}
		}
		if (!walls.isEmpty()) {
			// remove any walls left over after building the cell, without throwing an exception. 
//			throw new ImproperlyStructuredCellWallsException();
		}

		Cell cell = new Cell(id, group, connectedWalls);
		cell.setTruncated(truncated);
		return cell;
	}

	private Wall buildWall(int id, Element wallElement) {
		boolean edgewall = Boolean.parseBoolean(wallElement
				.getAttribute("edgewall"));
		int group = Integer.parseInt(wallElement.getAttribute("group"));
		LinkedList<Point> points = new LinkedList<Point>();
		NodeList pointNodes = ((Element) wallElement.getElementsByTagName(
				"points").item(0)).getElementsByTagName("point");
		for (int i = 0; i < pointNodes.getLength(); i++) {
			Node node = pointNodes.item(i);
			if (node instanceof Element) {
				Element pointElement = (Element) node;
				points.add(buildPoint(pointElement));
			}
		}
		Wall wall = new Wall(points);
		wall.setEdgeWall(edgewall);
		wall.setGroup(group);
		wall.setId(id);
		return wall;
	}

	private Point buildPoint(Element pointElement) {
		int x = (int)Float.parseFloat(pointElement.getAttribute("x"));
		int y = (int)Float.parseFloat(pointElement.getAttribute("y"));
		return new Point(x, y);
	}

	private int getCellCount() {
		NodeList cellss = doc.getElementsByTagName("cells");
		Element cells = (Element) cellss.item(0);
		int count = Integer.parseInt(cells.getAttribute("count"));
		return count;
	}
}
