package rootgui.model.recordstore;

import java.util.ArrayList;
import java.util.LinkedList;

import rootgui.model.SimulationRun;
import rootgui.model.objects.Species;

public class RecordStore implements IRecordStore {

	protected SimulationRun sr;
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -7048509525416088119L;
	
	
	private ArrayList<Record> records;
	
	private transient java.util.Map<Integer, RecordStoreActionListener> listeners;

	public RecordStore(SimulationRun sr) {
		this.sr = sr;
//		this.simID = simID;
		records = new ArrayList<Record>();
		listeners = new java.util.HashMap<Integer, RecordStoreActionListener>();
	}

	@Override
	public int setGeometry(String xmlFile, int iteration) {
		Record r = records.get(iteration);
		r.setGeometryFromFile(xmlFile);
		return 0;
	}

	@Override
	public Record getRecord(int iteration) {
		return records.get(iteration);
	}

	public void removeRecord(int iteration) {
		records.remove(iteration);
		fireRecordDeleted(iteration);
	}

	@Override
	public void removeRecordsStartingFrom(int startingIteration) {
		for (int i = records.size() - 1; i >= (0 > startingIteration ? 0
				: startingIteration); i--) {
			records.remove(i);
		}
	}

	@Override
	public SpeciesRecord getSpeciesRecord(Species species, int iteration) {
		return records.get(iteration).getData(species);
	}

	@Override
	public void addRecord(Record r) {
		r.setStore(this, records.size());
		records.add(r);
		fireRecordAdded(records.size() - 1);		
	}

	@Override
	public void fireRecordAdded(int iteration) {
		for (RecordStoreActionListener l : listeners.values()) {
			l.fireRecordAdded(iteration);
		}
	}

	@Override
	public void fireRecordDeleted(int iteration) {
		for (RecordStoreActionListener l : listeners.values()) {
			l.fireRecordDeleted(iteration);
		}
	}

	@Override
	public void fireRecordUpdated(int iteration) {
		for (RecordStoreActionListener l : listeners.values()) {
			l.fireRecordUpdated(iteration);
		}
	}

	@Override
	public void addListener(RecordStoreActionListener l) {
		if (listeners==null) {
			listeners=new java.util.HashMap<Integer, RecordStoreActionListener>();
		}		
		if (!listeners.containsKey(l.hashCode()))
			listeners.put(l.hashCode(), l);
	}

	@Override
	public void removeListener(RecordStoreActionListener l) {
		listeners.remove(l);
	}

	@Override
	public int size() {
		return records.size();
	}

	@Override
	public void clear() {
		for (int i = records.size() - 1; i >= 0; i--) {
			Record r = records.get(i);
			r.detachFromStore();
			records.remove(i);
			fireRecordDeleted(i);
		}
	}

	@Override
	public Record updateRecord(Record updatedRecord) {
		int index = updatedRecord.getIteration();
		updatedRecord.setStore(this, index);
		try {
			Record oldRecord = records.get(index);
			if (updatedRecord.getGeometry() != null)
				oldRecord.setGeometry(updatedRecord.getGeometry());
			for (SpeciesRecord sr : updatedRecord.getData()) {
				oldRecord.setRecord(sr);
			}
			fireRecordUpdated(index);
			return oldRecord;
		} catch (Exception e) {
			if (index < 0) index = 0;
			while (index >= records.size()) {				
				records.add(new Record(this));
			}
			records.set(index, updatedRecord);
			// e.printStackTrace();
			fireRecordUpdated(index);
			return updatedRecord;
		}
	}

	@Override
	public int getSimID() {
		return sr.getSimID();
	}
	
	@Override
	public SimulationRun getSimulationRun() {
		return sr;
	}

	@Override
	public void fireCellDataUpdated(Record r, String cellID, String speciesName,
			double value) {
		for (RecordStoreActionListener l : listeners.values()) {
			l.fireCellSpeciesUpdated(r, cellID, speciesName, value);
		}
	}
}
