package rootgui.view;

import java.io.Serializable;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import rootgui.model.Parameter;
import rootgui.model.objects.Cell;
import rootgui.model.objects.Species;
import rootgui.model.objects.Species.Type;
import rootgui.model.recordstore.Record;
import rootgui.model.recordstore.SpeciesRecord;
import visualiser.viewpanel.view.CellView;

public class CellInformationTableModel extends AbstractTableModel implements Serializable{

	Record record;
	/*
	 * 
	 */
	Cell cell;

	Vector<Object[]> tableData;

	/*
	 * 
	 */
	private boolean updatingRecord;

	public CellInformationTableModel() {

		tableData = new Vector<Object[]>();
	}

	public void clear(){
		this.record = null;
		this.cell = null;
		tableData.clear();
		fireTableDataChanged();
	}
	/*
	 * 
	 */
	public void updateRecord() {
		if (record != null) {
			for (int i = 2; i < tableData.size(); i++) { // No need to update
															// CellId or
															// GroupName
				Object[] row = tableData.get(i);
				Species species = (Species) row[0];
				Double v = Double.parseDouble(row[1].toString());
				record.getData(species).put(cell.getId(), v);
			}
		}
	}

	public boolean isCellEditable(int row, int col) {
		return col == 1 && row > 1;
	}

	public void setValueAt(Object value, int row, int col) {
		if (row < getRowCount() && col < getColumnCount()) {
			if (col == 1) {
				double newV = Double.parseDouble(String.valueOf(value));
				double oldV = (Double) tableData.get(row)[col];
				if (newV != oldV) {
					tableData.get(row)[col] = newV;
					fireTableCellUpdated(row, col);
					if (isUpdatingRecord()) {
						SpeciesRecord sr =  record.getData((Species) tableData.get(row)[0]);
						sr.put(cell.getId(), newV);
					}
				}
			}

		}
	}

	public Cell getCell(){
		return cell;
	}
	
	public void setCell(Record record, Cell cell) {		
		this.record = record;
		this.cell = cell;
		tableData.clear();
		Object[] rowData;

		if (record == null || cell == null) {
			fireTableDataChanged();
			return;
		}
		// add cell id to the table
		rowData = new Object[2];
		rowData[0] = "Cell ID";
		rowData[1] = cell.getId();
		tableData.add(rowData);

		// add cell group to the table
		rowData = new Object[2];
		rowData[0] = "Cell type";
		rowData[1] = record.getGeometry().getCellGroupNameById(cell.getGroup());
//		System.out.println("Group: "+cell.getGroup()+" "+rowData[1]);
		tableData.add(rowData);

		for (Species s : record.getSpecies()) {// Species.Type.CELL
			try {
				rowData = new Object[2];
				if (s.getType() == Type.CELL) {
					SpeciesRecord sr = record.getData(s);
					if(sr!=null){
					rowData[0] = s;
					rowData[1] = sr.get(cell.getId());
					tableData.add(rowData);
					}
				}
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		fireTableDataChanged();
	}

	@Override
	public int getColumnCount() {
		return 2;
	}

	@Override
	public int getRowCount() {
		return tableData.size();
	}

	@Override
	public Object getValueAt(int arg0, int arg1) {
		if (tableData.get(arg0)[arg1] instanceof Double && isUpdatingRecord() == true)
			return String.format("%.2f", (Double)tableData.get(arg0)[arg1]);
		if (tableData.get(arg0)[arg1] instanceof Species)
			return ((Species) tableData.get(arg0)[arg1]).getPublicName();
		return tableData.get(arg0)[arg1];
	}

	public boolean isUpdatingRecord() {
		return updatingRecord;
	}

	public void setUpdatingRecord(boolean updatingRecord) {
		this.updatingRecord = updatingRecord;
	}

}
