/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package rootgui.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.Rectangle;
import java.awt.ScrollPane;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import rootgui.controller.AbstractController;
import rootgui.controller.RunnableBlockingQueueListActionListener;
import rootgui.controller.VRootController;
import rootgui.model.Parameter;
import rootgui.model.SimulationRecord;
import rootgui.model.SimulationRun;
import rootgui.model.TabPropertyChangeEvent;
import rootgui.model.objects.Cell;
import rootgui.model.objects.SimulationModel;
import rootgui.model.objects.Species;
import rootgui.model.recordstore.Record;
import rootgui.view.settingspanel.*;
import simserver.AbsSimServerClient;
import visualiser.Figure;
import visualiser.IVisualiser;
import visualiser.VisualiserActionListener;
import visualiser.viewpanel.controller.ViewPanelActionListener;

/**
 * 
 * @author Teodor Ghetiu. Based on Sun's MVC example.
 */
public class DisplayViewFrame extends AbstractViewFrame implements
		ViewPanelActionListener, DefaultSettingsPanelActionListener,
		LeahsSettingsPanelActionListener, VisualiserActionListener,
		RunnableBlockingQueueListActionListener, ActionListener {

	final String default_steps = "20";
	final String default_timestep = "100";

	final JPanel thisFrame;

	private JButton jbRunSim, jbNewSim;

	static final String newLine = "\n";

	static final String wikiUrl = "https://sourceforge.net/p/virtualroot/wiki/Home/";

	// private RunnableBlockingQueueList serverTasks;

	/*
	 * This indicates the level to be printed to the console window
	 */
	private int coLevel = BIOLOGIST;

	/*
	 * For loading and saving simulations
	 */
	private JFileChooser fileChooser;

	/*
	 * 
	 */
	private JComponent visPanel;

	private IVisualiser vis;

	/*
	 * 
	 */
	// private Map userSettings;
	private boolean isConnected = false;

	/*
	 * A reference to the main VRootController of the RootGUI
	 */
	private AbstractController controller;

	/*
	 * A reference to the SettingsPanel
	 */
	private AbstractSettingsPanel settingsPanel;

	/*
	 * Visual Components
	 */
	JPanel pnParameters;
	JPanel pnSimParameters;
	JPanel pnModParameters;
	JTextPane tpConsole;

	// Pop up menu
	JPopupMenu popup;

	// DefaultListModel<SimulationModel> dlmModels;
	JTextField jtfIterations;
	JTextField jtfTimestep;

	// table model to display information of the selected cell
	// CellInformationTableModel cellInfoTableModel;

	// label to indicate whether the simulation is in progress
	JLabel progressLabel;
	/*
	 * Icon showing that server communication is on
	 */
	ImageIcon iconServerCommOn;
	/*
	 * Icon showing that server communication is off
	 */
	ImageIcon iconServerCommOff;
	static final String progressLabelIconPath1 = "images/loading_in_progress.gif";
	static final String progressLabelIconPath2 = "images/loading_static.png";

	private List<JMenu> menus;
	JMenuItem jmiLoadSim, jmiSaveSim, jmiNewSim;
	JPanel jpCellInfo;
	JTable jtCellInfo;
	private Map<Integer, Map<String, Serializable>> configurationMaps;
	private Map<Integer, Thread> executionThreads = new HashMap<Integer, Thread>();

	private int oldSimID = -1;

	public DisplayViewFrame(VRootController controller, IVisualiser vis) {
		super();
		thisFrame = this;
		this.controller = controller;
		this.vis = vis;
		this.visPanel = vis.getRenderPanel();
		configurationMaps = new HashMap<Integer, Map<String, Serializable>>();

		// serverTasks = new RunnableBlockingQueueList("DVF", this);
		initComponents();

		// userSettings = controller.getUserSettings();
		//
		renderUserSettings();
	}

	@Override
	public List<JMenu> getMenus() {
		return menus;
	}

	/**
	 * Initialization method called from the constructor
	 */
	private void initComponents() {
		// setTitle("VirtualRoot Simulator");
		setLayout(new BorderLayout());
		setPreferredSize(new Dimension(1024, 668));

		/* PARAMETERS PANEL */
		pnParameters = new JPanel(new GridBagLayout());
		// pnParameters.setBackground(Color.RED);
		GridBagConstraints c = new GridBagConstraints();

		// setting panel
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 4;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		if (settingsPanel == null) {
			// settingsPanel = new DefaultSettingsPanel(this, this);
			settingsPanel = new LeahsSettingsPanel(this, this);
		}
		settingsPanel.setMinimumSize(new Dimension(300, 300));
		// settingsPanel.setBackground(Color.GREEN);
		pnParameters.add(settingsPanel, c);
		// pnParameters.setBackground(Color.RED);
		// Run & New Simulation buttons
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 2;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		jbRunSim = new JButton("Run");
		jbRunSim.setActionCommand("Run");
		jbRunSim.setToolTipText("Execute the simulation.");
		jbRunSim.setEnabled(false);
		jbRunSim.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				// new Thread(new Runnable() {
				//
				// @Override
				// public void run() {

				// settingsPanelMaps.set(simID,
				// getCurrentConfigurationMap());
				final int simID = vis.getSelectedSimID();
				Map<String, Serializable> map = configurationMaps.get(simID);

				if (jbRunSim.getActionCommand().equals("Run")) {
					jbRunSim.setText("Pause");
					map.put("jbRunSim.getText()", jbRunSim.getText());
					jbRunSim.setActionCommand("Pause");
					map.put("jbRunSim.getActionCommand()",
							jbRunSim.getActionCommand());
					jbRunSim.setToolTipText("Pause the simulation");
					map.put("jbRunSim.getToolTipText()",
							jbRunSim.getToolTipText());
					settingsPanel.setEnabled(false);
					map.put(AbstractSettingsPanel.ENABLED,
							settingsPanel.isEnabled());
					// jbNewSim.setEnabled(true);
					// map.put("jbNewSim.isEnabled()", jbNewSim.isEnabled());
					if (jbRunSim.getText().equals("Continue"))// &&
						printlnStd(vis.getSelectedSimID(),
								"Continuing simulation", ANY);
					else
						printlnStd(vis.getSelectedSimID(),
								"\nExecuting simulation", ANY);

					fireRunSimPressed(simID);
				} else if (jbRunSim.getActionCommand().equals("Pause")) {
					firePauseSimPressed(simID);
				}

				// }
				// }).start();
			}
		});
		pnParameters.add(jbRunSim, c);

		c.gridx = 2;
		jbNewSim = new JButton("New");
		jbNewSim.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				fireNewSimPressed(vis.getSelectedSimID());
			}
		});
		jbNewSim.setEnabled(false);
		jbNewSim.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				// TODO Auto-generated method stub
				jmiNewSim.setEnabled(jbNewSim.isEnabled());
			}
		});
		pnParameters.add(jbNewSim, c);

		// simulation in progress label
		c.gridx = 0;
		c.gridy = 2;
		c.gridwidth = 4;
		c.weightx = 1;
		c.weighty = 0;
		iconServerCommOn = createImageIcon(progressLabelIconPath1);
		iconServerCommOff = createImageIcon(progressLabelIconPath2);
		progressLabel = new JLabel(iconServerCommOff);
		progressLabel.setBackground(Color.RED);
		// progressLabel.setPreferredSize(new Dimension(250, 50));
		progressLabel.setHorizontalAlignment(JLabel.CENTER);
		pnParameters.add(progressLabel, c);

		// iterations
		c.gridx = 0;
		c.gridy = 3;
		c.gridwidth = 1;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.LINE_START;
		c.weightx = 0;
		pnParameters.add(new JLabel("# steps: "), c);

		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		c.gridx = 1;
		c.gridy = 3;
		c.gridwidth = 1;
		jtfIterations = new JTextField(default_steps);
		jtfIterations.setEnabled(false);
		jtfIterations.setToolTipText("Number of time steps to simulate");
		jtfIterations.addFocusListener(new FocusListener() {
			int oldValue = Integer.parseInt(default_steps);

			@Override
			public void focusLost(FocusEvent e) {
				// TODO add validation and colour changing if problems appear
				if (!jtfIterations.isEditable())
					return;

				final int simID = vis.getSelectedSimID();
				Runnable r = new Runnable() {
					@Override
					public String toString() {
						return "jtfIterations.focusLost()";
					}

					@Override
					public void run() {
						Map<String, Serializable> map = configurationMaps
								.get(simID);
						try {
							int value = Integer.parseInt(jtfIterations
									.getText());
							if (value < controller.getIteration(simID)) {
								value = controller.getIteration(simID);
								jtfIterations.setText(value + "");
								map.put("jtfIterations.getText()", value);

							} else if (value > controller
									.getMaxIterations(simID)
									&& (controller.getIteration(simID) == (controller
											.getMaxIterations(simID)))) {
								if (jbRunSim.isEnabled() == false) {
									map.put("jbRunSim.getText()", "Continue");
									map.put("jbRunSim.isEnabled()",
											new Boolean(true));
									map.put("jbRunSim.getActionCommand()",
											"Run");
									map.put(AbstractSettingsPanel.ENABLED,
											new Boolean(true));
									if (simID == vis.getSelectedSimID()) {
										jbRunSim.setText("Continue");
										jbRunSim.setEnabled(true);
										jbRunSim.setActionCommand("Run");
									}
								}
							}
							if (value != oldValue) {
								oldValue = value;
								controller.setMaxIterations(simID, oldValue);
							}
						} catch (NumberFormatException e1) {
							jtfIterations.setText(oldValue + "");
							map.put("jtfIterations.setText()", oldValue);
						}
					}
				};
				new Thread(r).start();
			}

			@Override
			public void focusGained(FocusEvent e) {
				// TODO Auto-generated method stub
			}
		});
		pnParameters.add(jtfIterations, c);

		// timestep
		c.gridx = 2;
		c.gridy = 3;
		c.ipady = 0;
		c.insets = new Insets(0, 0, 0, 0);
		c.gridwidth = 1;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		pnParameters.add(new JLabel("Step length: "), c);

		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		c.gridx = 3;
		c.gridy = 3;
		c.gridwidth = 1;
		c.anchor = GridBagConstraints.LINE_START;
		jtfTimestep = new JTextField(default_timestep);
		jtfTimestep.setEnabled(false);
		jtfTimestep.setToolTipText("Duration of each step (s)");
		jtfTimestep.addFocusListener(new FocusListener() {
			double oldValue = Double.parseDouble(default_timestep);

			@Override
			public void focusLost(FocusEvent e) {
				if (!jtfTimestep.isEditable() || !jtfTimestep.isEnabled())
					return;

				double value = oldValue;
				int simID = vis.getSelectedSimID();
				Map<String, Serializable> map = configurationMaps.get(simID);
				try {
					value = Double.parseDouble(jtfTimestep.getText());
					if (value < 0) {
						value = 0.1;
						jtfTimestep.setText(value + "");
						map.put("jtfTimestep.setText()", value);
					}
				} catch (NumberFormatException e1) {
					map.put("jtfTimestep.setText()", oldValue);
					jtfTimestep.setText(oldValue + "");
				}

				controller.setTimestep(simID, value);
			}

			@Override
			public void focusGained(FocusEvent e) {
				// TODO Auto-generated method stub
			}
		});
		// jtfTimestep.setPreferredSize(new Dimension(35, 25));
		pnParameters.add(jtfTimestep, c);

		/* VISUALISATION PANEL */
		visPanel.setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder("Visualisation"),
				BorderFactory.createEmptyBorder(5, 5, 5, 5)));

		// tpConsole.setText("");

		/* CONSOLE OUTPUT PANEL */
		tpConsole = new JTextPane();
		tpConsole.setEditable(false);

		/* POPUP MENUS */
		final JPopupMenu popup = new JPopupMenu();
		JMenuItem menuItem = new JMenuItem("Clear");
		menuItem.addActionListener(this);
		popup.add(menuItem);

		tpConsole.addMouseListener(new MouseAdapter() {

			public void mousePressed(MouseEvent e) {
				maybeShowPopup(e);
			}

			public void mouseReleased(MouseEvent e) {
				maybeShowPopup(e);
			}

			public void maybeShowPopup(MouseEvent e) {
				if (e.isPopupTrigger()) {
					popup.show(e.getComponent(), e.getX(), e.getY());
				}
			}
		});

		// tpConsole.addMouseListener(new PopupListener());

		// @Override
		// public void mouseReleased(MouseEvent e) {
		// // TODO Auto-generated method stub
		// maybeShowPopup(e);
		// }
		//
		// @Override
		// public void mousePressed(MouseEvent e) {
		//
		// maybeShowPopup(e);
		// }
		//

		tpConsole.addMouseListener(new MouseListener() {

			@Override
			public void mouseReleased(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mousePressed(MouseEvent e) {
				// TODO Auto-generated method stub
				PopupMenu pm = new PopupMenu("");
				MenuItem mi = new MenuItem("Clea&r");
				mi.addActionListener(new ActionListener() {

					@Override
					public void actionPerformed(ActionEvent e) {
						// TODO Auto-generated method stub
						tpConsole.setText("");
					}
				});
				pm.add(mi);
			}

			@Override
			public void mouseExited(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mouseEntered(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mouseClicked(MouseEvent e) {
				// TODO Auto-generated method stub

			}
		});
		JScrollPane spConsole = new JScrollPane(tpConsole);
		spConsole
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);

		// panel to display cell information and whether a simulation is in
		// progess
		// cell information panel
		jpCellInfo = new JPanel(new GridLayout(0, 1));
		JPanel cellInfoPanel = new JPanel(new GridLayout(0, 1));
		// cellInfoTableModel = new CellInformationTableModel();
		// cellInfoTableModel.addTableModelListener(new TableModelListener() {
		//
		// @Override
		// public void tableChanged(TableModelEvent arg0) {
		// if (cellInfoTableModel.isUpdatingRecord()) {
		// cellInfoTableModel.updateRecord();
		// }
		// }
		// });
		jtCellInfo = new JTable(new CellInformationTableModel());
		// cellInformationTable.setPreferredSize(new Dimension(230, 70));
		ScrollPane scrollCellInformationTable = new ScrollPane();
		scrollCellInformationTable.add(jtCellInfo);
		scrollCellInformationTable.setPreferredSize(new Dimension(200, 70));
		cellInfoPanel.add(scrollCellInformationTable);
		cellInfoPanel
				.setBorder(BorderFactory.createCompoundBorder(
						BorderFactory
								.createTitledBorder("Information of the selected cell"),
						BorderFactory.createEmptyBorder(5, 5, 5, 5)));
		jpCellInfo.add(cellInfoPanel);

		Dimension minSize = new Dimension(0, 0);

		/* PUTTING THEM TOGETHER */
		// lower part
		// pnLower = new JTabbedPane();
		// pnLower.setLayout(new BorderLayout());
		// pnLower.addTab("Output", null, spConsole);
		// upper left part
		JSplitPane spUpperLeft = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				pnParameters, jpCellInfo);
		spUpperLeft.setOneTouchExpandable(true);
		spUpperLeft.setDividerLocation(400);
		pnParameters.setMinimumSize(minSize);
		jpCellInfo.setMinimumSize(minSize);
		// spUpperLeft.setResizeWeight(0);

		// upper part
		JSplitPane spUpper = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				spUpperLeft, visPanel);
		// spUpper.setOneTouchExpandable(true);
		spUpper.setResizeWeight(0.5);
		spUpper.setDividerLocation(253);

		spUpperLeft.setMinimumSize(minSize);
		visPanel.setMinimumSize(minSize);

		// main
		JSplitPane spMain = new JSplitPane(JSplitPane.VERTICAL_SPLIT, spUpper,
				spConsole);
		spMain.setOneTouchExpandable(true);
		spMain.setResizeWeight(0.8);
		spMain.setDividerLocation(540);

		add(spMain, BorderLayout.CENTER);

		menus = createMenus();

		// setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		// setPreferredSize(new Dimension(700, 700));
		// pack();

		setVisible(true);

		// printStd(vis.getSelectedSimID(), "GUI initialised");
	}

	private List<JMenu> createMenus() {
		this.fileChooser = new JFileChooser();
		fileChooser.addChoosableFileFilter(new FileNameExtensionFilter(
				"SimulationRun files", "sr"));

		/* MENU */
		// JMenuBar jmb = new JMenuBar();
		ArrayList<JMenu> menuList = new ArrayList<JMenu>();
		JMenu fileMenu = new JMenu("File");

		jmiNewSim = new JMenuItem("New Simulation");
		jmiNewSim.setEnabled(false);
		jmiNewSim.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				// TODO Auto-generated method stub
				if (jbNewSim.isEnabled()) {
					if (JOptionPane
							.showConfirmDialog(thisFrame,
									"Are you sure you want to start a new simulation? You may loose existing data.") == JOptionPane.OK_OPTION)
						fireNewSimPressed(vis.getSelectedSimID());
				}
			}
		});
		fileMenu.add(jmiNewSim);

		jmiSaveSim = new JMenuItem("Save Simulation");
		jmiSaveSim.setEnabled(false);
		jmiSaveSim.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				int simID = vis.getSelectedSimID();
				SimulationRun simRun = controller.getSimulationRun(simID);
				Figure figure = vis.getFigure(simID);
				Map<String, Serializable> configuration = getCurrentConfigurationMap();
				if (simRun == null || configuration == null) {
					printlnStd(simID,
							"There is no SimulationRun or Configuration at present!");
					return;
				}

				if (fileChooser.showSaveDialog(thisFrame) == JFileChooser.APPROVE_OPTION) {
					SimulationRecord simRecord = new SimulationRecord(simRun,
							configuration, figure);
					File file = fileChooser.getSelectedFile();
					FileOutputStream fos = null;
					ObjectOutputStream out = null;
					try {
						fos = new FileOutputStream(file);
						out = new ObjectOutputStream(fos);
						out.writeObject(simRecord);
						out.close();
					} catch (IOException ex) {
						ex.printStackTrace();
					}

					// This is where a real application would save the file.
					printlnStd(simID, "\nSimuation saved to: " + file.getName());
				}
			}
		});
		fileMenu.add(jmiSaveSim);

		jmiLoadSim = new JMenuItem("Load Simulation");
		jmiLoadSim.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				Runnable r = new Runnable() {

					@Override
					public void run() {
						int returnVal = fileChooser.showOpenDialog(thisFrame);

						if (returnVal == JFileChooser.APPROVE_OPTION) {
							int simID = vis.getSelectedSimID();
							File file = fileChooser.getSelectedFile();
							printlnStd(simID, "\nLoading simulation from "
									+ file.getName() + " ...");

							// load data
							SimulationRecord simRecord = null;
							FileInputStream fis = null;
							ObjectInputStream in = null;
							try {
								fis = new FileInputStream(file);
								in = new ObjectInputStream(fis);
								simRecord = (SimulationRecord) in.readObject();
								in.close();
								printlnStd(simID, "Simulation loaded.");
							} catch (IOException ex) {
								ex.printStackTrace();
								printErr(simID, "Error loading simulation.\n");
							} catch (ClassNotFoundException ex) {
								ex.printStackTrace();
							}

							setConfigurationMap(simRecord.getConfiguration());
							controller.setSimulationRun(simID,
									simRecord.getSimulationRun());
							int maxiumIterations = simRecord.getSimulationRun()
									.getMaxIterations();

							// visualise
							vis.resetTab(simID);
							vis.setFigure(simID, simRecord.getFigure());
							vis.setMaxIterations(vis.getSelectedSimID(),
									maxiumIterations);
							vis.setRenderedIteration(vis.getSelectedSimID(),
									simRecord.getSimulationRun().getRecord()
											.getIteration());
							settingsPanel.setEnabled(true);

							printlnStd(vis.getSelectedSimID(),
									"Load simulation from: " + file.getName()
											+ "." + newLine);
						}
					}
				};
				new Thread(r).start();
			}
		});
		fileMenu.add(jmiLoadSim);
		// JMenuItem exit = new JMenuItem("Exit");
		// exit.addActionListener(new ActionListener() {
		//
		// @Override
		// public void actionPerformed(ActionEvent e) {
		// // thisFrame.dispose();
		// }
		// });
		// fileMenu.add(exit);

		JMenu helpMenu = new JMenu("Help");
		JMenuItem userManual = new JMenuItem("User Manual");
		userManual.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {

				try {
					java.awt.Desktop.getDesktop().browse(
							java.net.URI.create(wikiUrl));
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		});

		// public static void main(String[] args) {
		// try {
		// //Set your page url in this string. For eg, I m using URL for Google
		// Search engine
		// String url = "http://www.google.com";
		// java.awt.Desktop.getDesktop().browse(java.net.URI.create(url));
		// }
		// catch (java.io.IOException e) {
		// System.out.println(e.getMessage());
		// }
		// }
		//

		helpMenu.add(userManual);

		menuList.add(fileMenu);
		menuList.add(helpMenu);
		// jmb.add(jm);
		// setJMenuBar(jmb);
		return menuList;
	}

	/*
	 * Specifies if the application (VRootController) is connected to the server
	 * or not
	 */
	private void setServerConnected(final int simID, final boolean onOff,
			final boolean showWarningDialog) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				// Turn off metal's use of bold fonts
				// UIManager.put("swing.boldMetal", Boolean.FALSE);
				Map<String, Serializable> map = configurationMaps.get(simID);
				map.put(AbstractSettingsPanel.ENABLED, new Boolean(onOff));
				if (onOff) {
					// TODO: enable some components
					if (simID == vis.getSelectedSimID())
						settingsPanel.setEnabled(true);
					printlnStd(simID, "Server connected successfully.", ANY);
				} else { // no server connection
					// TODO: disable some components
					map.put("jbRunSim.setEnabled()", new Boolean(false));
					map.put("jbNewSim.setEnabled()", new Boolean(false));
					if (simID == vis.getSelectedSimID()) {
						settingsPanel.setEnabled(false);
						jbRunSim.setEnabled(false);
						jbNewSim.setEnabled(false);
					}
					if (showWarningDialog) {
						printErr(simID, "Server connection failed.", ANY);
						JOptionPane.showMessageDialog(thisFrame,
								"Connection to server lost!", "Warning",
								JOptionPane.WARNING_MESSAGE);
					}
				}
				// enableServerComControls(onOff);
			}
		});
	}

	/*
	 * Groups the visual controls that should be enabled in case a server
	 * connection is available (and disabled otherwise).
	 */
	// private void enableServerComControls(boolean onOff) {
	//
	// }
	/*
	 * 
	 */
	public void setServerCommAnimation(final int simID, final boolean onOff) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				// Turn off metal's use of bold fonts
				// UIManager.put("swing.boldMetal", Boolean.FALSE);
				if (configurationMaps.containsKey(simID)) {
					Map<String, Serializable> map = configurationMaps
							.get(simID);
					map.put("setServerCommAnimation", onOff);
				}
				if (vis.getSelectedSimID() == simID || simID == -1)
					progressLabel.setIcon(onOff ? iconServerCommOn
							: iconServerCommOff);
			}
		});
	}

	public void modelPropertyChange(TabPropertyChangeEvent evt) {
		final int simID = evt.getSimID();
		settingsPanel.modelPropertyChange(evt);
		if (evt.getPropertyName().equals(
				VRootController.AVAILABLE_SIMULATION_MODELS_CHANGED)) {
			// reload simulation models
		} else if (evt.getPropertyName().equals(VRootController.TIMESTEP)) {
			String val = ((Double) evt.getNewValue()).toString();
			Map<String, Serializable> map = configurationMaps.get(simID);
			map.put("jtfTimestep.getText()", val);
			if (vis.getSelectedSimID() == simID)
				jtfTimestep.setText(val);
		} else if (evt.getPropertyName().equals(VRootController.MAX_ITERATIONS)) {
			final Integer iterationNum = (Integer) evt.getNewValue();
			Map<String, Serializable> map = configurationMaps.get(simID);
			map.put("jtfIterations.getText()", iterationNum + "");
			if (vis.getSelectedSimID() == simID)
				jtfIterations.setText(iterationNum + "");
			Runnable r = new Runnable() {
				@Override
				public String toString() {
					return "MAX_ITERATIONS";
				}

				@Override
				public void run() {
					vis.setMaxIterations(simID, iterationNum);
				}
			};
			if (controller.hasSimID(simID))
				new Thread(r).start();
		} else if (evt.getPropertyName().equals(VRootController.MODEL_LOADED)) {
			printlnStd(simID, "Model <" + evt.getNewValue().toString()
					+ "> loaded in the simulation.", DEBUG);
		} else if (evt.getPropertyName().equals(VRootController.INC_ITERATION)) {
			final Record r = (Record) evt.getNewValue();

			Runnable rb = new Runnable() {
				@Override
				public String toString() {
					return "INC_ITERATION (" + r.getIteration() + ")";
				}

				@Override
				public void run() {
					vis.setRecord(simID, r, false);
					printlnStd(simID, "Step " + r.getIteration() + " ready.",
							BIOLOGIST);
				}
			};
			if (controller.hasSimID(simID))
				new Thread(rb).start();
		} else if (evt.getPropertyName().equals(
				VRootController.ITERATION_UPDATED)) {
			final Record rc = (Record) evt.getNewValue();
			Runnable r = new Runnable() {
				@Override
				public String toString() {
					return "ITERATION_UPDATED (" + simID + ", "
							+ rc.getIteration() + ")";
				}

				@Override
				public void run() {
					vis.setRecord(simID, rc, false);
					printlnStd(simID, "Iteration updated for tab " + simID
							+ ": " + rc.getIteration(), DEBUG);
				}
			};
			if (controller.hasSimID(simID))
				SwingUtilities.invokeLater(r);
		} else if (evt.getPropertyName().equals(
				VRootController.SIMULATION_INITIALISED)) {
			// TODO assuming models have already been loaded
			// vis.setRunEnabled(vis.getSelectedSimID(), true);
			Runnable rb = new Runnable() {
				@Override
				public String toString() {
					return "SIMULATION_INITIALISED(" + vis.getSelectedSimID()
							+ ")";
				}

				@Override
				public void run() {
					Species[] loadedSpecies = controller
							.getLoadedSpeciesArray(simID);
					// vis.setFigure(simID, new Figure(loadedSpecies));
					vis.setLoadedSpecies(simID, loadedSpecies);
					vis.setRecord(simID, controller.getRecord(simID), false);
					vis.setMaxIterations(simID,
							Integer.parseInt(jtfIterations.getText()));

					Map<String, Serializable> map = configurationMaps
							.get(simID);
					map.put("jbRunSim.isEnabled()", new Boolean(true));
					map.put("jbNewSim.isEnabled()", new Boolean(true));
					map.put("jtfIterations.isEditable()", new Boolean(true));
					map.put("jtfIterations.isEnabled()", new Boolean(true));
					map.put("jtfTimestep.isEditable()", new Boolean(true));
					map.put("jtfTimestep.isEnabled()", new Boolean(true));
					if (vis.getSelectedSimID() == simID) {
						jbRunSim.setEnabled(true);
						jbNewSim.setEnabled(false);
						jtfIterations.setEditable(true);
						jtfIterations.setEnabled(true);
						jtfTimestep.setEditable(true);
						jtfTimestep.setEnabled(true);
					}
				}
			};
			new Thread(rb).start();
			// serverTasks.addRunnableToQueue(simID, rb);

		} else if (evt.getPropertyName().equals(
				VRootController.TISSUE_FILE_INITIALISED)) {
			printlnStd(simID, "Tissue file initialized", DEBUG);
			vis.setRecord(simID, controller.getRecord(simID), true);
		}
		// else if (evt.getPropertyName().equals(
		// VRootController.SIMULATION_IN_PROGRESS)) {
		// // setServerCommAnimation(evt.getsimID(), ((Boolean)
		// // evt.getNewValue()).booleanValue());
		// }
		else if (evt.getPropertyName().equals(VRootController.SSC_CONNECTED)) {
			setServerConnected(simID,
					((Boolean) evt.getNewValue()).booleanValue(), true);
		}
		// else if (evt.getPropertyName().equals(//
		// VRootController.SIMULATION_RUN_REMOVED)) {
		// }
		else if (evt.getPropertyName().equals(VRootController.RECORD_UPDATED)) {
			int iteration = (Integer) evt.getNewValue();
			vis.fireRecordUpdated(simID, iteration);
			printlnStd(simID, "Information updated.", ANY);
		}
	}

	public void setVisualiser(JPanel visualiserPanel) {
		this.visPanel = visualiserPanel;
	}

	/*
	 * Renders visually the user settings
	 */
	private void renderUserSettings() {

	}

	// @Override
	// public void actionPerformed(ActionEvent arg0) {
	// SimulationRun sr = controller.getSimulationRun(vis
	// .getSelectedTabIndex());
	// // if (arg0.getSource() == null) {
	// // sr.setTissueFileName(null);
	// // } else if (arg0.getSource() == null) {
	// // sr.setModelParameter(null, null, null);
	// // } else if (arg0.getSource() == null) {
	// // sr.loadSimulationModelnull);
	// // } else if (arg0.getSource() == null) {
	// // controller.unloadSimulationModel(vis.getSelectedTabIndex(),null);
	// // }
	// // else if (arg0.getSource() == null) {
	// // sr.getModelDefaultParameters(null).get(null);
	// // } else if (arg0.getSource() == null) {
	// // sr.setMaxIterations(0);
	// // }
	// }

	/** * IConsoleOutput ** */
	@Override
	public void setOutputLevel(int level) {
		this.coLevel = level;
	}

	@Override
	public int getOutputLevel() {
		return coLevel;
	}

	@Override
	public void printStd(final int simID, String message) {
		printStd(simID, message, ANY);
	}

	public void printStd(final int simID, String message, int messageLevel) {
		if (messageLevel >= coLevel) {
			SimpleAttributeSet keyWord = new SimpleAttributeSet();
			StyleConstants.setForeground(keyWord, Color.BLACK);
			StyleConstants.setBackground(keyWord, Color.WHITE);
			// StyleConstants.setItalic(keyWord, true);

			StyledDocument doc = null;
			try {
				doc = (StyledDocument) configurationMaps.get(simID).get(
						"tpConsole.getStyledDocument()");
			} catch (Exception e1) {
				doc = tpConsole.getStyledDocument();
			}
			try {
				doc.insertString(doc.getLength(), message, keyWord);
				scrollDownConsoleOutput();
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}
	}

	@Override
	public void printlnStd(final int simID, String message) {
		printlnStd(simID, message, ANY);
	}

	@Override
	public void printlnStd(final int simID, String message, int messageLevel) {
		if (coLevel <= messageLevel) {
			SimpleAttributeSet keyWord = new SimpleAttributeSet();
			StyleConstants.setForeground(keyWord, Color.BLACK);
			StyleConstants.setBackground(keyWord, Color.WHITE);
			// StyleConstants.setItalic(keyWord, true);

			StyledDocument doc = null;
			try {
				doc = (StyledDocument) configurationMaps.get(simID).get(
						"tpConsole.getStyledDocument()");
				// System.err.println(doc);
			} catch (Exception e1) {
				doc = tpConsole.getStyledDocument();
			}
			try {
				doc.insertString(doc.getLength(), "\n" + message, keyWord);
				scrollDownConsoleOutput();
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}
	}

	@Override
	public void printErr(final int simID, String message) {
		printErr(simID, message, ANY);
	}

	@Override
	public void printErr(final int simID, String message, int messageLevel) {
		if (coLevel <= messageLevel) {
			SimpleAttributeSet keyWord = new SimpleAttributeSet();
			StyleConstants.setForeground(keyWord, Color.RED);
			StyleConstants.setBackground(keyWord, Color.WHITE);
			// StyleConstants.setItalic(keyWord, true);

			StyledDocument doc = null;
			try {
				doc = (StyledDocument) configurationMaps.get(simID).get(
						"tpConsole.getStyledDocument()");
			} catch (Exception e1) {
				doc = tpConsole.getStyledDocument();
			}
			try {
				doc.insertString(doc.getLength(), "\n" + message, keyWord);
				scrollDownConsoleOutput();
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}
	}

	@Override
	public void printException(final int simID, Exception e) {
		SimpleAttributeSet keyWord = new SimpleAttributeSet();
		StyleConstants.setForeground(keyWord, Color.RED);
		StyleConstants.setBackground(keyWord, Color.WHITE);
		StyleConstants.setItalic(keyWord, true);

		StyledDocument doc = null;
		try {
			doc = (StyledDocument) configurationMaps.get(simID).get(
					"tpConsole.getStyledDocument()");
		} catch (Exception e1) {
			doc = tpConsole.getStyledDocument();
		}
		try {
			doc.insertString(doc.getLength(), "\n" + e.getMessage(), keyWord);
			scrollDownConsoleOutput();
		} catch (BadLocationException ex) {
			ex.printStackTrace();
		}
	}

	private void scrollDownConsoleOutput() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				Rectangle visibleArea = tpConsole.getVisibleRect();
				int coHeight = tpConsole.getBounds().height;
				visibleArea.y = coHeight - 1;// visibleArea.height;
				tpConsole.scrollRectToVisible(visibleArea);
			}
		});
	}

	// public void setIsConnected(boolean connected) {
	// isConnected = connected;
	// if (isConnected) {
	// // enable buttons, etc.
	// } else {
	// // disable buttons, etc.
	// }
	// }

	@Override
	public void fireCellSelected(final int simID, final Record record,
			final Cell cell) {
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				if (simID == vis.getSelectedSimID()) {
					CellInformationTableModel cellInfoTableModel = (CellInformationTableModel) jtCellInfo
							.getModel();
					cellInfoTableModel.setUpdatingRecord(false);
					cellInfoTableModel.setCell(record, cell);
					cellInfoTableModel.setUpdatingRecord(true);
				} else {
					Map<String, Serializable> map = configurationMaps
							.get(simID);
					map.put("selectedCell", new Object[] { record, cell });
				}
			}
		});

	}

	@Override
	public void fireCellUnselected(final int simID) {
		if (simID == vis.getSelectedSimID()) {
			CellInformationTableModel cellInfoTableModel = (CellInformationTableModel) jtCellInfo
					.getModel();
			cellInfoTableModel.setUpdatingRecord(false);
			cellInfoTableModel.setCell(null, null);
			cellInfoTableModel.setUpdatingRecord(true);
		} else {
			Map<String, Serializable> map = configurationMaps.get(simID);
			if (map.containsKey("selectedCell"))
				map.remove("selectedCell");
		}
	}

	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = DisplayViewFrame.class.getResource(path);
		ImageIcon icon = null;
		if (imgURL != null) {
			icon = new ImageIcon(imgURL);
			return icon;
		} else {
			System.err.println("Couldn't find file: " + path);
			return icon;
		}
	}

	@Override
	public void fireSetButtonPressed(final SimulationModel model) {
		final int simID = vis.getSelectedSimID();
		// Runnable r = new Runnable() {
		// @Override
		// public String toString() {
		// return "fireSetButtonPressed(" + model + ")";
		// }
		//
		// @Override
		// public void run() {
		controller.loadSimulationModel(simID, model);
		// }
		// };
		// serverTasks.addRunnableToQueue(simID, r);
		// setServerCommAnimation(false);
	}

	/*
	 * Called when a Initialise Simulation button is pressed
	 * 
	 * @see rootgui.view.setting.DefaultSettingsPanelActionListener#
	 * fireInitSimButtonPressed()
	 */
	@Override
	public void fireInitSimButtonPressed() {
		final int simID = vis.getSelectedSimID();
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "fireInitSimButtonPressed(" + simID + ")";
			}

			@Override
			public void run() {

				// if (controller.getTissueFileName(simID) == null) {
				// controller.setTissueFileName(simID, "R12data_OA_wc.zip");
				// controller.initTissue(simID);
				// }

				int iterationNum = 10;
				try {
					iterationNum = Integer.parseInt(jtfIterations.getText());
					controller.setMaxIterations(simID, iterationNum);
				} catch (NumberFormatException e1) {
					// e1.printStackTrace();
				}
				//

				try {
					controller.setTimestep(simID,
							Double.parseDouble(jtfTimestep.getText()));
				} catch (NumberFormatException e) {
					// TODO Auto-generated catch block
					// e.printStackTrace();
				}

				controller.setSimulationInitialised(simID);
			}
		};
		// serverTasks.addRunnableToQueue(simID, r);
		new Thread(r).start();
	}

	@Override
	public void fireGeometryChanged(String geometryFileName) {
		final int simID = vis.getSelectedSimID();
		printlnStd(simID, "\nInitialising root geometry...", ANY);
		boolean fileChanged = false;
		if (geometryFileName == null)
			geometryFileName = "";
		boolean geometryFileOk = (geometryFileName.length() > 4);
		boolean controllerFileDiff = (controller.getTissueFileName(simID) == null || !controller
				.getTissueFileName(simID).equals(geometryFileName));
		if (geometryFileOk) {
			if (controllerFileDiff) {
				controller.setTissueFileName(simID, geometryFileName);
				fileChanged = true;
			}
			if (!controller.getSimulationRun(simID).isTissueFileInitialized()
					|| fileChanged)
				controller.initTissue(simID);
		}
		// }
		// };
		// // serverTasks.addRunnableToQueue(simID, r);
	}

	@Override
	public void fireParameterChanged(final String modelName,
			final Parameter param) {
		final int simID = vis.getSelectedSimID();
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "fireParameterChanged()";
			}

			@Override
			public void run() {
				controller.setModelParameter(simID, modelName, param.getName(),
						param.getValue());
				printlnStd(simID, "Parameter " + param.getName()
						+ " changed to " + param.getValue());
			}
		};
		// serverTasks.addRunnableToQueue(simID, r);
		new Thread(r).start();
	}

	@Override
	public void fireSetGrowth(final boolean onOff) {
		final int simID = vis.getSelectedSimID();
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "fireSetGrowth()";
			}

			@Override
			public void run() {
				controller.setGrowth(simID, onOff);
				printlnStd(simID, "Growth set to " + onOff);
			}
		};
		// serverTasks.addRunnableToQueue(simID, r);
		new Thread(r).start();
	}

	@Override
	public void fireCarriersChanged(final String[] groups, final boolean[] onOff) {
		final int simID = vis.getSelectedSimID();
		printlnStd(simID, "Setting auxin carriers...");
		// Runnable r = new Runnable() {
		// @Override
		// public String toString() {
		// return "fireCarriersChanged()";
		// }
		//
		// @Override
		// public void run() {
		// //TODO error checking?
		controller.setCellCarrier(simID, groups, onOff);
		// }
		// };
		// // serverTasks.addRunnableToQueue(simID, r);
		// new Thread(r).start();
	}

	@Override
	public void fireNewSimPressed(final int simID) {
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "fireNewSimPressed()";
			}

			@Override
			public void run() {
				printlnStd(simID, "\nStarting new simulation...");
				// "The method newSimulation(int) is undefined for the type AbstractController, then throws exception"
				controller.newSimulation(simID);
				vis.resetTab(simID);
				controller.resetSimulation(simID);
				if (simID == vis.getSelectedSimID()) {
					jbRunSim.setText("Run");
					jbRunSim.setActionCommand("Run");
					settingsPanel.setEnabled(true);
					jbNewSim.setEnabled(false);
					// jtfIterations.setText("10");
					((CellInformationTableModel) jtCellInfo.getModel()).clear();
					settingsPanel.reset();
					settingsPanel.triggerNewSimulation();
				} else {
					Map<String, Serializable> map = configurationMaps
							.get(simID);
					map.put("jbRunSim.getText()", "Run");
					map.put("jbRunSim.isEnabled()", new Boolean(true));
					map.put("jbRunSim.getActionCommand()", "Run");
					map.put("jbNewSim.isEnabled()", new Boolean(false));
					map.put(AbstractSettingsPanel.ENABLED, new Boolean(true));
					// map.put("jtfIterations.setText()", "10");
					map.put(AbstractSettingsPanel.TRIGGER_NEW_SIMULATION,
							new Boolean(true));
				}
			}
		};
		// // serverTasks.addRunnableToQueue(simID, r);
		new Thread(r).start();
	}

	@Override
	public void firePauseSimPressed(final int simID) {
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				Map<String, Serializable> map = configurationMaps.get(simID);
				// map.put(AbstractSettingsPanel.ENABLED, true);
				map.put("jbRunSim.getText()", "Run");
				map.put("jbRunSim.getActionCommand()", "Run");
				map.put("jbRunSim.isEnabled()", true);
				map.put("jbRunSim.getToolTipText()", "Execute the simulation");
				if (vis.getSelectedSimID() == simID) {
					// settingsPanel.setEnabled(true);
					jbRunSim.setText("Run");
					jbRunSim.setActionCommand("Run");
					jbRunSim.setEnabled(true);
					jbRunSim.setToolTipText("Execute the simulation");
				}
			}
		});

	}

	@Override
	public void fireRunSimPressed(final int simID) {
		Map<String, Serializable> map = configurationMaps.get(simID);
		map.put("jbRunSim.getActionCommand()", "Pause");
		settingsPanel.setEnabled(false);

		Thread t = new Thread(new Runnable() {
			@Override
			public String toString() {
				return "fireRunSimPressed(" + simID + ")";
			}

			@Override
			public void run() {
				try {
					Map<String, Serializable> map = configurationMaps
							.get(simID);
					boolean b1 = controller.getIteration(simID) < controller
							.getMaxIterations(simID);
					boolean b2 = map.get("jbRunSim.getActionCommand()").equals(
							"Pause");

					if (simID == vis.getSelectedSimID()) {
						jtfIterations.setEditable(false);
						jtfTimestep.setEditable(false);
					} else {
						map.put("jtfIterations.isEditable()",
								new Boolean(false));
						map.put("jtfTimestep.isEditable()", new Boolean(false));
					}
					int i = 0;
					while (b1 && b2) {
						if (controller.hasSimID(simID)) {
							controller.step(simID);
							Thread.sleep(1);
							b1 = controller.getIteration(simID) < controller
									.getMaxIterations(simID);
							System.err.println(i + ". after step: b1 = " + b1);
							b2 = map.get("jbRunSim.getActionCommand()").equals(
									"Pause");
						} else
							b1 = b2 = false;
						i++;
					}
					if (simID == vis.getSelectedSimID()) {
						if (controller.getIteration(simID) >= controller
								.getMaxIterations(simID))
							jbRunSim.setEnabled(false);
						jtfIterations.setEditable(true);
						jtfTimestep.setEditable(true);
						jbRunSim.setText("Run");
						jbRunSim.setActionCommand("Run");
						jbNewSim.setEnabled(true);
						jmiSaveSim.setEnabled(true);
					} else {
						if (controller.getIteration(simID) >= controller
								.getMaxIterations(simID))
							map.put("jbRunSim.isEnabled()", false);
						map.put("jtfIterations.isEditable()", true);
						map.put("jtfTimestep.isEditable()", true);
						map.put("jbRunSim.getText()", "Run");
						map.put("jbRunSim.getActionCommand()", "Run");
						map.put("jbNewSim.isEnabled()", jbNewSim.isEnabled());
						map.put("jmiSaveSim.isEnabled()", true);
					}
				} catch (InterruptedException ex) {
					Thread.currentThread().interrupt();
					System.err.println("DVF interrupted");
					return;
				} catch (Exception e) {
					return;
				}
			}
		});
		t.start();
		executionThreads.put(simID, t);
	}

	private void setConfigurationMap(Map<String, Serializable> map) {
		settingsPanel.setConfigurationMap(map);

		jbRunSim.setEnabled((Boolean) map.get("jbRunSim.isEnabled()"));
		jbRunSim.setText((String) map.get("jbRunSim.getText()"));
		jbRunSim.setActionCommand(((String) map
				.get("jbRunSim.getActionCommand()")));

		jbNewSim.setEnabled((Boolean) map.get("jbNewSim.isEnabled()"));
		jbNewSim.setText((String) map.get("jbNewSim.getText()"));
		jbNewSim.setActionCommand(((String) map
				.get("jbNewSim.getActionCommand()")));

		jtfTimestep.setEnabled((Boolean) map.get("jtfTimestep.isEnabled()"));
		jtfTimestep.setEditable((Boolean) map.get("jtfTimestep.isEditable()"));
		jtfTimestep.setText((String) map.get("jtfTimestep.getText()"));

		jtfIterations
				.setEnabled((Boolean) map.get("jtfIterations.isEnabled()"));
		jtfIterations.setText((String) map.get("jtfIterations.getText()"));
		jtfIterations.setEditable((Boolean) map
				.get("jtfIterations.isEditable()"));

		int simID = vis.getSelectedSimID();
		setServerCommAnimation(simID,
				(Boolean) map.get("setServerCommAnimation"));
		tpConsole.setStyledDocument((StyledDocument) map
				.get("tpConsole.getStyledDocument()"));

		CellInformationTableModel cellInfoModel = ((CellInformationTableModel) map
				.get("cellInfoModel"));
		jtCellInfo.setModel(cellInfoModel);
		cellInfoModel.fireTableDataChanged();
		if (map.containsKey("selectedCell")) {
			Object[] o = (Object[]) map.get("selectedCell");
			cellInfoModel.setUpdatingRecord(false);
			cellInfoModel.setCell((Record) o[0], (Cell) o[1]);
			cellInfoModel.setUpdatingRecord(true);
		}

		jmiSaveSim.setEnabled((Boolean) map.get("jmiSaveSim.isEnabled()"));
	}

	private Map<String, Serializable> getCurrentConfigurationMap() {
		Map<String, Serializable> map = settingsPanel.getConfigurationMap();
		// todo add run/new sim button enabled, edit, timestep, snapshots
		map.put("jbRunSim.isEnabled()", jbRunSim.isEnabled());
		map.put("jbRunSim.getText()", jbRunSim.getText());
		map.put("jbRunSim.getActionCommand()", jbRunSim.getActionCommand());

		map.put("jbNewSim.isEnabled()", jbNewSim.isEnabled());
		map.put("jbNewSim.getText()", jbNewSim.getText());
		map.put("jbNewSim.getActionCommand()", jbNewSim.getActionCommand());

		map.put("jtfTimestep.isEnabled()", jtfTimestep.isEnabled());
		map.put("jtfTimestep.isEditable()", jtfTimestep.isEditable());
		map.put("jtfTimestep.getText()", jtfTimestep.getText());

		map.put("jtfIterations.isEnabled()", jtfIterations.isEnabled());
		map.put("jtfIterations.isEditable()", jtfIterations.isEditable());
		map.put("jtfIterations.getText()", jtfIterations.getText());
		map.put("setServerCommAnimation",
				progressLabel.getIcon().equals(iconServerCommOn));
		map.put("tpConsole.getStyledDocument()",
				(Serializable) tpConsole.getStyledDocument());
		// oldMap.put("tpConsole.getText()", tpConsole.getText());
		CellInformationTableModel cellInfoModel = (CellInformationTableModel) jtCellInfo
				.getModel();
		map.put("cellInfoModel", cellInfoModel);

		// menus
		map.put("jmiSaveSim.isEnabled()", jmiSaveSim.isEnabled());
		return map;
	}

	private void resetCurrentConfiguration() {
		settingsPanel.reset();
		settingsPanel.setEnabled(true);

		jbRunSim.setEnabled(false);
		jbRunSim.setText("Run");
		jbRunSim.setActionCommand("Run");

		jbNewSim.setEnabled(false);
		jbNewSim.setText("New");
		jbNewSim.setActionCommand("New");

		jtfTimestep.setEnabled(false);
		jtfTimestep.setText(default_timestep);

		jtfIterations.setEnabled(false);
		jtfIterations.setText(default_steps);

		setServerCommAnimation(-1, false);
		// buttons and stuff
		tpConsole.setStyledDocument(new DefaultStyledDocument());
		CellInformationTableModel cellInfoModel = new CellInformationTableModel();
		jtCellInfo.setModel(cellInfoModel);
		cellInfoModel.fireTableDataChanged();

		// menus
		jmiSaveSim.setEnabled(false);
	}

	/*
	 * This gets fired after the new tab has actually been selected
	 * 
	 * @see visualiser.VisualiserActionListener#fireTabSelected(int, int)
	 */
	@Override
	public void fireTabSelected(final int tabIndex, final int simID) {
		if (simID == oldSimID)
			return;

		// System.out.println("DVF.fireTabSelected (" + oldSimID + ", " + simID+
		// ")");
		try {
			// if (oldIndex == vis.getSelectedTabIndex()) {
			Map<String, Serializable> oldMap = getCurrentConfigurationMap();
			configurationMaps.put(oldSimID, oldMap);
			// }
		} catch (Exception e) {
			System.err
					.println("Error in DisplayViewFrame.fireTabSelected: oldSimID = "
							+ oldSimID
							+ " / "
							+ configurationMaps.size()
							+ " is incorrect");
		}

		try {
			Map<String, Serializable> map = configurationMaps.get(simID);
			setConfigurationMap(map);
			settingsPanel.setSimID(simID);
		} catch (Exception e) {
			System.err
					.println("Error in DisplayViewFrame.fireTabSelected: simID = "
							+ simID
							+ " / "
							+ configurationMaps.size()
							+ " is incorrect");
		}
		oldSimID = simID;
	}

	@Override
	public void fireTabAdded(final int tabIndex, final int simID) {
		// System.out.println("DVF.fireTabAdded (" + tabIndex + ", " + simID +
		// ")");
		if (tabIndex > 0 && oldSimID >= 0) // configurationMaps.size() >
											// oldSimID &&
			configurationMaps.put(oldSimID, getCurrentConfigurationMap());

		if (tabIndex > 0)
			resetCurrentConfiguration();
		settingsPanel.setSimID(simID);

		configurationMaps.put(simID, getCurrentConfigurationMap());
		oldSimID = simID;

		// if (oldIndex != -1)
		// setConfigurationMap(configurationMaps.get(oldIndex > 0 ? oldIndex :
		// 0));

		// Runnable r = new Runnable() {
		// @Override
		// public String toString() {
		// return "fireTabAdded()";
		// }
		//
		// @Override
		// public void run() {
		controller.createSimulationRun(simID);
		// // if (oldIndex >= 0) {
		// // setConfigurationMap(settingsPanelMaps.get(oldIndex));
		// // }
		// }
		// };
		// serverTasks.createQueue();
	}

	@Override
	public void fireTabRemoved(final int tabIndex, final int simID) {
		new Thread(new Runnable() {

			@Override
			public void run() {
				try {
					Thread t = (Thread) executionThreads.get(simID);
					t.interrupt();
					executionThreads.remove(simID);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				controller.removeSimulationRun(simID);
				configurationMaps.remove(simID);
			}
		}).start();
	}

	@Override
	public void fireJobFinished(final int simID, final int jobID,
			final int remainingJobCount) {
		// int count = serverTasks.getQueuedJobsCount(simID);
		setServerCommAnimation(simID, remainingJobCount != 0);
	}

	@Override
	public void fireJobAdded(int simID, int jobID, int remainingJobCount) {
		setServerCommAnimation(simID, true);
	}

	@Override
	public void destroy() {
		// serverTasks.removeAllQueues();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		JMenuItem source = (JMenuItem) (e.getSource());
		StyledDocument doc = null;
		doc = tpConsole.getStyledDocument();
		try {
			doc.remove(0, doc.getLength());
		} catch (BadLocationException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		try {
			doc.insertString(doc.getLength(), "Console cleared", null);
		} catch (BadLocationException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	}

}
