package rootgui.view;

import javax.swing.*;

import rootgui.model.objects.Species;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Point2D;
import java.util.Arrays;
import java.util.HashMap;

public class SpeciesSelectionDialog extends JDialog implements ActionListener {
	private static SpeciesSelectionDialog dialog;
	private static Species[] species;
	private static int[] selectedRows;
	private static double[] minRanges;
	private static double[] maxRanges;
	/*
	 * Specifies the number of columns the user wants to have, in order to render the species
	 */
	private static int noColumns = 0;
	private static HashMap<Species, Point2D.Double> speciesValues = new HashMap<Species, Point2D.Double>();
	private JTable table;
	private JSpinner spinner;
		
	public final static int OK = 0;
	public final static int CANCEL = 1;

	private static int buttonAction = CANCEL;

	public static Species[] showDialog(Component frameComp,
			Component locationComp) {
		Frame frame = JOptionPane.getFrameForComponent(frameComp);
		dialog = new SpeciesSelectionDialog(frame, locationComp, "Species",
				"Select species to visualise", null, null, null);
		dialog.setVisible(true);
		return species;
	}

	/**
	 * Set up and show the dialog. The first Component argument determines which
	 * frame the dialog depends on; it should be a component in the dialog's
	 * controlling frame. The second Component argument should be null if you
	 * want the dialog to come up with its left corner in the center of the
	 * screen; otherwise, it should be the component on top of which the dialog
	 * should appear.
	 */
	public static Species[] showDialog(Component frameComp,
			Component locationComp, String labelText, String title,
			Species[] possibleValues) {
		Frame frame = JOptionPane.getFrameForComponent(frameComp);
		Arrays.sort(possibleValues);
		dialog = new SpeciesSelectionDialog(frame, locationComp, labelText,
				title, possibleValues, null, null);
		dialog.setVisible(true);
		return species;
	}

	public static int[] getSelectedRows() {
		return selectedRows;
	}

	public static Species[] getSpecies() {
		return species;
	}

	public static double[] getMinRanges() {
		return minRanges;
	}

	public static double[] getMaxRanges() {
		return maxRanges;
	}
	
	public static int getButtonAction(){
		return buttonAction;
	}
	
	public static int getNoColumns(){
		return noColumns;
	}

	private SpeciesSelectionDialog(Frame frame, Component locationComp,
			String labelText, String title, Species[] data, double[] minRanges,
			double[] maxRanges) {
		super(frame, title, true);
		species = data;
		
		if (minRanges != null)
			SpeciesSelectionDialog.minRanges = minRanges;
		if (maxRanges != null)
			SpeciesSelectionDialog.maxRanges = maxRanges;

		// Create and initialize the buttons.
		JButton setButton = new JButton("OK");

		setButton.setActionCommand("Set");
		setButton.addActionListener(this);
		//
		final JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(this);

		getRootPane().setDefaultButton(setButton);

		String[] columnNames = { "Species"}; //, "Min", "Max" };
//		Object[][] tableData = new Object[data.length][3];
		Object[][] tableData = new Object[data.length][1];
		for (int i = 0; i < data.length; i++) {
			tableData[i][0] = data[i].getPublicName();
			Point2D.Double pd = speciesValues.get(data[i]);
//			tableData[i][1] = minRanges != null ? minRanges[i] : (pd != null ? pd.getX() : 0);
//			tableData[i][2] = maxRanges != null ? maxRanges[i]  : (pd != null ? pd.getY() : 0);
		}

		table = new JTable(tableData, columnNames);

		JScrollPane scrollPane = new JScrollPane(table);
		table.setFillsViewportHeight(true);
		scrollPane.setPreferredSize(new Dimension(300, 80));
		
		SpinnerModel spinnerModel = new SpinnerNumberModel(1, //initial value
                1, //min
                5, //max
                1);  
		spinner = new JSpinner(spinnerModel);
		// Create a container so that we can add a title around
		// the scroll pane. Can't add a title directly to the
		// scroll pane because its background would be white.
		// Lay out the label and scroll pane from top to bottom.
		JPanel listPane = new JPanel();
		listPane.setLayout(new BoxLayout(listPane, BoxLayout.PAGE_AXIS));
		JLabel label = new JLabel(labelText);
		label.setLabelFor(table);
		listPane.add(label);
		listPane.add(Box.createRigidArea(new Dimension(0, 5)));
		listPane.add(scrollPane);
		listPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		// Lay out the buttons from left to right.
		JPanel buttonPane = new JPanel();
		buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.LINE_AXIS));
		buttonPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
		buttonPane.add(Box.createHorizontalGlue());
		JLabel colLabel = new JLabel("Column NO.");
		colLabel.setToolTipText("Please specify the number of columns to be displayed");
		buttonPane.add(colLabel);
//		buttonPane.add(field);
		buttonPane.add(spinner);
		buttonPane.add(Box.createRigidArea(new Dimension(10, 0)));
		buttonPane.add(setButton);
		buttonPane.add(Box.createRigidArea(new Dimension(10, 0)));
		buttonPane.add(cancelButton);

		// Put everything together, using the content pane's BorderLayout.
		Container contentPane = getContentPane();
		contentPane.add(listPane, BorderLayout.CENTER);
		contentPane.add(buttonPane, BorderLayout.PAGE_END);

		pack();
		setLocationRelativeTo(locationComp);
	}

	// Handle clicks on the Set and Cancel buttons.
	public void actionPerformed(ActionEvent e) {
		if ("Set".equals(e.getActionCommand())) {
			selectedRows = table.getSelectedRows();

			// int selectedCount = table.getSelectedRowCount();
			int rowCount = table.getRowCount();

//			species = new Species[rowCount];
			minRanges = new double[rowCount];
			maxRanges = new double[rowCount];
			for (int i = 0; i < rowCount; i++) {
//				species[i] = (Species) table.getModel().getValueAt(i, 0);
				// if (table.isRowSelected(i))
				// species[i].setSelected(true);
//				minRanges[i] = Double.parseDouble(table.getModel().getValueAt(i, 1).toString());
//				System.out.println(table.getModel().getValueAt(i, 2));
//				maxRanges[i] = Double.parseDouble(table.getModel().getValueAt(i, 2).toString());

//				speciesValues.put(species[i], new Point2D.Double(minRanges[i], maxRanges[i]));
				
				noColumns = Integer.parseInt(getTextField(spinner).getText());
			}
			buttonAction = OK;
		} else {
			buttonAction = CANCEL;
		}

		dialog.setVisible(false);
	}
	/**
     * Return the formatted text field used by the editor, or
     * null if the editor doesn't descend from JSpinner.DefaultEditor.
     */
    public JFormattedTextField getTextField(JSpinner spinner) {
        JComponent editor = spinner.getEditor();
        if (editor instanceof JSpinner.DefaultEditor) {
            return ((JSpinner.DefaultEditor)editor).getTextField();
        } else {
            System.err.println("Unexpected editor type: "
                               + spinner.getEditor().getClass()
                               + " isn't a descendant of DefaultEditor");
            return null;
        }
    }
}