package rootgui.view.settingspanel;

import java.beans.PropertyChangeEvent;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JPanel;

import visualiser.Figure;

public abstract class AbstractSettingsPanel extends JPanel {

	public final static String ENABLED = "AbstractSettingsPanel.ENABLED";
	public final static String TRIGGER_NEW_SIMULATION = "AbstractSettingsPanel.TRIGGER_NEW_SIMULATION";
	public final static String SIMULATION_INITIALISED = "AbstractSettingsPanel.SIMULATION_INITIALISED";
	
	/*
	 * ConfigurationMap
	 */
	protected Map<String, Serializable> map = new HashMap<String, Serializable>();
	/**
	 * Called by the controller when it needs to pass along a property change
	 * from a model.
	 * 
	 * @param evt
	 *            The property change event from the model
	 */
	public abstract void modelPropertyChange(PropertyChangeEvent evt);


	/**
	 * Get the figure object, if one has been chosen by the user;
	 * Note, this method is not currently in use in LeahsSettingsPanel
	 * @return Figure, the way in which the user want to view the simulation result
	 */
	public abstract Figure getFigure();

	/**
	 * Add a listener to monitor all actions from the settings panel
	 * @param l
	 */
	public abstract void addSettingPanelActionListener(
			DefaultSettingsPanelActionListener l);

	/**
	 * Remove an action listener
	 * @param l
	 */
	public abstract void removeSettingPanelActionListener(
			DefaultSettingsPanelActionListener l);
	
	/**
	 * Export current configurations that could be loaded back sometime
	 * @return a map object describing current configurations
	 */
	public Map<String, Serializable> getConfigurationMap(){
		map.put(ENABLED, isEnabled());
		return map;
	}
	
	/**
	 * Set current configurations of the settings panel to the one described in the incoming map object
	 * @param map, should be the one generated by the getConfigurationMap() method.
	 */
	public void setConfigurationMap(Map<String, Serializable> map){
//		if (this.map == map) return;
		this.map = map;
		setEnabled(((Boolean) map.get(ENABLED)).booleanValue());
		if (map.containsKey(TRIGGER_NEW_SIMULATION) && ((Boolean) map.get(TRIGGER_NEW_SIMULATION)).booleanValue()){
			triggerNewSimulation();
			map.remove(TRIGGER_NEW_SIMULATION);
		}
	}


	public abstract void triggerNewSimulation();
	
	public void reset(){
		map = new HashMap<String, Serializable>();		
	}

	protected int simID = -1;
	public void setSimID(int simID) {
		this.simID = simID;
	}
}
