package simserver;

import java.util.HashMap;
import java.util.Map;

import rootgui.model.Parameter;

public abstract class AbsSimServerClient implements ISimServerClient {

	public static boolean isTrue(String response) {
		return response != null && "true".equalsIgnoreCase(response);
	}

	/*
	 * 
	 */
	public static String[] parseStringArray(String pythonArray) {
		String[] result = null;
		if (pythonArray != null && pythonArray.length() > 0) {
			pythonArray = pythonArray.substring(1, pythonArray.length() - 1);

			result = pythonArray.split("[, ]+");
			for (int i = 0; i < result.length; i++) {
				result[i] = result[i].substring(1, result[i].length() - 1);
			} 
		} else {
			result = new String[]{};
		}
		return result;
	}

	/*
	 * Returns a float array parsed from a Python
	 */
	public static float[] parseFloatArray(String pythonArray) {
		pythonArray = pythonArray.substring(1, pythonArray.length() - 1);
		String[] sresult = pythonArray.split("[, ]+");
		float[] result = new float[sresult.length];
		for (int i = 0; i < result.length; i++) {
//			System.out.println(sresult[i]);
			result[i] = Float.parseFloat(sresult[i]);
		}
		return result;
	}

	/*
	 * Returns a float array parsed from a Python
	 */
	public static double[] parseDoubleArray(String pythonArray) {
		pythonArray = pythonArray.substring(1, pythonArray.length() - 1);
		String[] sresult = pythonArray.split("[, ]+");
		double[] result = new double[sresult.length];
		for (int i = 0; i < result.length; i++) {
			// System.out.println(sresult[i]);
			result[i] = Double.parseDouble(sresult[i]);
		}
		return result;
	}

	public static Map<String, String> parseStringDictionary(String response)
			throws Exception {
		Map<String, String> result = new HashMap<String, String>();
		try {
			if (response == null || response.length() <= 2)
				return result;
			String input = response.substring(1, response.length() - 1);
			// String regex = "[\"]{1}[^\".]*[\"]{1}";

			if (input.startsWith("\"") && input.endsWith("\"")) {
				input = input.substring(1, input.length() - 1);
			}
			String[] entries = input.split("\", \"");
			for (String entry : entries) {
				String[] ss = entry.split("\": \"");
//				assert ss.length == 2;
				// System.out.printf("%s -> %s\n", ss[0], ss[1]);
				result.put(ss[0], ss[1]);
			}
		} catch (Exception e) {
			e.printStackTrace();
			System.err.println("Server response: " + response);
		}
		return result;
	}

	public static Map<String, Double> parseStringDoubleDictionary(
			String response) throws Exception {
		Map<String, Double> result = new HashMap<String, Double>();
		if (response == null || response.length() <= 2)
			return result;
		String input = response.substring(1, response.length() - 1);
		// System.out.println(input);

		if (input.startsWith("\"") && input.endsWith("\"")) {
			input = input.substring(1, input.length() - 1);
		}
		String[] entries = input.split(",");
		int zeroCount = 0;
		for (String entry : entries) {
			String[] ss = entry.split(":");
			assert ss.length == 2;
			ss[0] = ss[0].trim();
			// System.out.printf("%s -> %s\n", ss[0], ss[1]);
			String k = ss[0].substring(1, ss[0].length() - 1).trim();
			double v = Double.parseDouble(ss[1].trim());
			if (v == 0) zeroCount++;
			result.put(k, v);
		}
//		System.out.println("ASC: pSDD : zeroCount = " + zeroCount);
		return result;
	}

	public static Map<Integer, Double> parseIntegerDoubleDictionary(
			String response) throws Exception {
		Map<Integer, Double> result = new HashMap<Integer, Double>();
		if (response == null || response.length() <= 2)
			return result;
		String input = response.substring(1, response.length() - 1);
		// System.out.println(input);

		// if (input.startsWith("\"") && input.endsWith("\"")) {
		// input = input.substring(1, input.length() - 1);
		// }
		String[] entries = input.split(",");
		for (String entry : entries) {
			String[] ss = entry.split(":");
			assert ss.length == 2;
			String atom1 = ss[0].trim();
			atom1 = atom1.startsWith("\"") ? atom1.substring(1,
					atom1.length() - 1) : atom1;
			String atom2 = ss[1].trim();
			atom2 = atom2.startsWith("\"") ? atom2.substring(1,
					atom2.length() - 1) : atom2;
			result.put(Integer.parseInt(atom1), Double.parseDouble(atom2));
		}
		return result;
	}

	/*
	 * 
	 */
	public static Map<String, Parameter> parseParamDictionary(String response)
			throws Exception {
		Map<String, Parameter> result = new HashMap<String, Parameter>();
		if (response == null || response.length() <= 2)
			return result;
		String input = response.substring(1, response.length() - 1);
		// System.out.println(input);

		if (input.startsWith("\"")) {
			input = input.substring(1, input.length());
		}
		String[] entries = input.split(", \"");
		for (String entry : entries) {
			String[] ss = entry.split("\":");
			assert ss.length == 2;
			String pName = ss[0];
			String pValue = ss[1].trim();
			result.put(pName, new Parameter(pName, pValue));
		}
		return result;
	}

	public static Map<Integer, Map<Integer, Double>> parseIntegerTupleDoubleDictionary(
			String response) {
		Map<Integer, Map<Integer, Double>> result = new HashMap<Integer, Map<Integer, Double>>();
		if (response == null || response.length() <= 2)
			return result;
		String input = response.substring(1, response.length() - 1);
		// System.out.println(input);

		// if (input.startsWith("\"") && input.endsWith("\"")) {
		// input = input.substring(1, input.length() - 1);
		// }
		String[] entries = input.split(",");
		for (String entry : entries) {
			String[] ss = entry.split(":");
			assert ss.length == 2;
			String atom1 = ss[0].trim();
			atom1 = atom1.startsWith("\"") ? atom1.substring(1,
					atom1.length() - 1) : atom1;
			atom1 = atom1.substring(1, atom1.length() - 1);
			String[] idAtoms = atom1.split("_");
			Integer id1 = new Integer(idAtoms[0]);
			Integer id2 = new Integer(idAtoms[1]);

			String atom2 = ss[1].trim();
			atom2 = atom2.startsWith("\"") ? atom2.substring(1,
					atom2.length() - 1) : atom2;
			if (!result.containsKey(id1))
				result.put(id1, new HashMap<Integer, Double>());
			result.get(id1).put(id2, Double.parseDouble(atom2));
		}
		return result;
	}

	public static String buildPythonDictionary(String[] keys, boolean[] values) {
		StringBuilder sb = new StringBuilder("{");
		for (int i = 0; i < keys.length; i++)
			sb.append("'" + keys[i] + "' : " + (values[i] ? "True" : "False")
					+ ",");
		sb.deleteCharAt(sb.length() - 1);
		sb.append("}");
		return sb.toString();
	}

	public static String buildPythonDictionary(String[] keys, String value) {
		StringBuilder sb = new StringBuilder("{");
		for (String key : keys)
			sb.append("'" + key + "' : '" + value + "',");
		sb.deleteCharAt(sb.length() - 1);
		sb.append("}");
		return sb.toString();
	}

	public static String buildPythonDictionary(String[] keys, String[] values) {
		StringBuilder sb = new StringBuilder("{");
		for (int i = 0; i < keys.length; i++)
			sb.append("'" + keys[i] + "' : '" + values[i] + "',");
		sb.deleteCharAt(sb.length() - 1);
		sb.append("}");
		return sb.toString();
	}
	
	public static String buildPythonDictionary(String[] keys, Double[] values) {
		StringBuilder sb = new StringBuilder("{");
		for (int i = 0; i < keys.length; i++)
			sb.append("'" + keys[i] + "' : " + values[i] + ",");
		sb.deleteCharAt(sb.length() - 1);
		sb.append("}");
		return sb.toString();
	}
	
	public static String buildStringDoublePythonDictionary(java.util.Map<String, Double> data) {
		StringBuilder sb = new StringBuilder("{");
		for (String key : data.keySet())
			sb.append("'" + key + "' : " + data.get(key) + ",");
		sb.deleteCharAt(sb.length() - 1);
		sb.append("}");
		return sb.toString();
	}

	public abstract void setSpeciesData(String speciesName, Map<String, Double> data); 
	
	public abstract Map<String, Double> getSpeciesData(String speciesName);

	/*
	 * 
	 */
	public abstract boolean setCellCarrier(String[] groups, boolean[] onOff);

	public abstract String[] getAvailableTissueNames();
}
