package simserver;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.math.BigInteger;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.sql.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import rootgui.model.Geometry;
import rootgui.model.Parameter;
import rootgui.model.objects.SimulationModel;
import rootgui.model.objects.Species;
import rootgui.model.recordstore.Record;
import rootgui.view.IConsoleOutput;
import rootgui.view.SystemOutputAdapter;

/*
 * 
 */
public class SimServerClient extends AbsSimServerClient {	
	// Communication
	protected Socket socket;
	protected InputStream in;
	protected OutputStream out;
	protected BufferedReader br;
	protected PrintWriter pw;

	protected String hostName;
	protected int port;
	int timeoutMs = 1500;
	// Simulation parameters
//	protected int maxIterations = 0;
//	protected int iteration = 0;
//	protected double timestep = 0;

	private IConsoleOutput ico;

	// public static void main(String[] args) {
	// try {
	// SimServerClient ssc = new SimServerClient();
	// ssc.test();
	// } catch (UnknownHostException e) {
	// // TODO Auto-generated catch block
	// e.printStackTrace();
	// } catch (Exception e) {
	// // TODO Auto-generated catch block
	// e.printStackTrace();
	// }
	// }
	//
	// public void test() throws Exception {
	// setTissueFile("R12data_OA_wc.zip");
	// getAvailableModelsNames();
	//
	// String selectedModel = "at_ncwd_R12";
	// addModelByName(selectedModel);
	// setParameter(selectedModel, "A2", "100");
	// Map<String, Parameter> parameters =
	// getModelDefaultParameters(selectedModel);
	// getParameter(selectedModel, "A2");
	//
	// String[] species = listSpecies();
	// //
	// initTissue();
	// // initSimulation();
	// setIterations(1);
	// step();
	//
	// getSpeciesData(species[0]);
	// getGeometry();
	// disconnect();
	// }

	/* .... */

	/*
	 * ID corresponding to the VisualiserTab for which this connection is
	 * running.
	 */
	private int id = 0;

	/*
	 * 
	 */
//	public SimServerClient(int id) throws UnknownHostException, IOException {
//		this(id, "localhost", 4446, new SystemOutputAdapter());
//	}
//
//	public SimServerClient(int id, IConsoleOutput ico)
//			throws UnknownHostException, IOException {
//		this(id, "localhost", 4446, ico);
//	}
//
//	public SimServerClient(int id, AbsSimServerClient sscParent, IConsoleOutput ico) throws UnknownHostException, IOException {
//		this(id, sscParent.getHostname(), sscParent.getPort(), ico);
//	}

	/*
	 * 
	 */
	public SimServerClient(int id, String host, int port, IConsoleOutput ico)
			throws UnknownHostException, IOException, SocketTimeoutException {
		hostName = host;
		this.port = port;
		this.id = id;
		reconnect();

		if (ico == null)
			this.ico = new SystemOutputAdapter();
		else
			this.ico = ico;
	}

	/*
	 * Returns the hostname of the server to which this client is connected
	 */
	public String getHostname() {
		if (socket != null)
			return ((java.net.InetSocketAddress) socket
					.getRemoteSocketAddress()).getHostName();
		return null;
	}

	/*
	 * Returns the port of the server to which this client is connected
	 */
	public int getPort() {
		if (socket != null)
			return ((java.net.InetSocketAddress) socket
					.getRemoteSocketAddress()).getPort();
		return -1;
	}

	/*
	 * Irreversibly disconnects the client from the Server.
	 */
	@Override
	public synchronized void disconnect() {
		try {
			in.close();
			out.close();
			socket.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		in = null;
		out = null;
		socket = null;
	}

	@Override
	public void reconnect() throws UnknownHostException, IOException {
		if (socket != null)
			disconnect();
		InetAddress addr = InetAddress.getByName(hostName);
		SocketAddress sockaddr = new InetSocketAddress(addr, port);
		socket = new Socket();
		socket.connect(sockaddr, timeoutMs);

		socket.setTcpNoDelay(true);
		socket.setKeepAlive(true);

		in = socket.getInputStream();
		out = socket.getOutputStream();

		br = new BufferedReader(new InputStreamReader(in));
		pw = new PrintWriter(new OutputStreamWriter(out), true);
		
		if (!authenticate()){
			disconnect();
			ico.printlnStd(id, "Client authentication failed!");
		}
	}

	
	private boolean authenticate(){		
		try {
			String s = readResponse();
			// decode server key
			long serverKey = Long.parseLong(s);
			
			// apply client key
			String clientKey = serverKey + "_mama";
			
			// encode key
			MessageDigest digest = MessageDigest.getInstance("SHA-256");
			byte[] hash = digest.digest(clientKey.getBytes("UTF-8"));
//			byte[] hash = ("" + clientKey).getBytes();//"UTF-8"
			
			// send key			
			sendCommand(bytesToHex(hash));
			
			// read server confirmation
			s = readResponse();
			return isTrue(s);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		return false;
	}

	public static String bytesToHex(byte[] bytes) {
        StringBuffer result = new StringBuffer();
        for (byte byt : bytes) result.append(Integer.toString((byt & 0xff) + 0x100, 16).substring(1));
        return result.toString();
    }
	/*
	 * 
	 * @see simserver.AbsSimServerClient#isConnected()
	 */
	public boolean isConnected() {
		return socket != null && !socket.isClosed() && socket.isConnected();
	}

	private void sendCommand(String command) {
		sendCommand(command, true);
	}

	/*
	 * 
	 */
	private void sendCommand(String command, boolean printOutput) {
		pw.print(command + "\r\n");
		pw.flush();
		if (printOutput)
			System.out.println("command: " + command);
	}

	private String readResponse() {
		return readResponse(false);
	}

	/*
	 * 
	 */
	private String readResponse(boolean printOutput) {
		String response = null;
		try {

			response = br.readLine();
			if (response == null) {
				System.err.println("null response!!!");
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (printOutput)
			System.out.println("response: " + response);
		return response;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simserver.AbsSimServerClient#setTissueFile(java.lang.String)
	 */
	public boolean setTissueFile(String tissueFile) {
		sendCommand("s.set_tissue_name('" + tissueFile + "')");

		if (!tissueFile.equals(readResponse())) {
			ico.printErr(id, "SimServerClient: setTissueFile failed\n");
			return false;
		}
		return true;
	}

	/*
	 * 
	 * @see
	 * simserver.AbsSimServerClient#addModel(rootgui.model.objects.SimulationModel
	 * )
	 */
	public boolean addModel(SimulationModel sm) {
		sendCommand("s.add_model_by_name('" + sm.getName() + "')");

		if (!isTrue(readResponse())) {
			ico.printErr(id,
					"SimServerClient.addModel(): failed at add_model_by_name() stage\n");// ,
																							// IConsoleOutput.DEBUG
			return false;
		} else {
			// upload all parameters
			for (String parName : sm.getParameters().keySet()) {
				setParameter(sm.getName(), parName,
						sm.getParameters().get(parName).getValue());
			}
		}
		return true;
	}

	@Override
	public boolean addEditedModel(SimulationModel editedModel,
			SimulationModel defaultModel) {
		sendCommand("s.add_model_by_name('" + editedModel.getName() + "')");

		if (!isTrue(readResponse())) {
			ico.printErr(id,
					"SimServerClient.addModel(): failed at add_model_by_name() stage\n");
			return false;
		} else {
			// add only modified parameters
			for (String parName : editedModel.getParameters().keySet()) {
				if (!editedModel.getParameterValue(parName).equals(defaultModel.getParameterValue(parName)))
					setParameter(editedModel.getName(), parName, editedModel.getParameters().get(parName).getValue());
			}
		}

		return true;
	}

	public void addModelByName(String modelName) {
		sendCommand("s.add_model_by_name('" + modelName + "')");

		if (!isTrue(readResponse())) {
			ico.printErr(id, "SimServerClient.addModelByName(): failed\n");
		}
	}

	public void setSBMLModel(String fileName) {
		sendCommand("s.add_SBML_model('" + fileName + "')");

		if (!isTrue(readResponse())) {
			ico.printErr(id, "SimServerClient.setSBMLModel(): failed\n");
		}
	}

	public void unsetModel(String modelName) {
		sendCommand("s.remove_model_by_name('" + modelName + "')");
	}

	@Override
	public String[] getAvailableModelsNames() throws IOException {
		if (socket == null)
			return null;
		sendCommand("s.list_available_models()");

		return parseStringArray(readResponse());
	}

	@Override
	public boolean initTissue() {
		sendCommand("s.initialize_tissue()");

		if (!isTrue(readResponse())) {
			System.out.println("Init tissue failed");
			return false;
		}
		return true;
	}

	@Override
	public boolean initSimulation() {
		String command = "s.init_simulation()";
		sendCommand(command);

		return isTrue(readResponse(false));
	}

	/*
	 * Returns the species loaded in the simulation
	 */
	public String[] listSpecies() throws IOException {
		String command = "s.list_species()";
		sendCommand(command);

		return parseStringArray(readResponse());
	}
	
	public boolean setGrowth(boolean onOff)  {
		// TODO have it turning off?
//		System.err.println("Growth about to be set");
		String command = "s.add_module('growth')";
		sendCommand(command);

		return true;
	}
	
	public boolean setParameter(String modelName, String parameter, String value) {
		if (modelName != null && modelName.length() > 0)
			parameter = modelName + "." + parameter;
		sendCommand("s.set_parameter('" + parameter + "', '" + value + "')");

		String s = readResponse();
		// ico.printlnStd(s);
		return true;
	}

	public String getParameter(String modelName, String parameter) {
		if (modelName != null && modelName.length() > 0)
			parameter = modelName + "." + parameter;
		sendCommand("s.get_parameter('" + parameter + "')");

		return readResponse();
	}

	public float getNumParameter(String modelName, String parameter) {
		if (modelName != null && modelName.length() > 0)
			parameter = modelName + "." + parameter;
		sendCommand("s.get_parameter('" + parameter + "')");

		return Float.parseFloat(readResponse());
	}

	/*
	 * Sets a numerical parameter
	 */
	public void setNumParameter(String parameter, double value) {
		sendCommand("s.set_parameter('" + parameter + "', " + value + ")");

		String s = readResponse();
		 System.out.println(s);
	}

	public int setIterations(int iterations) {
//		maxIterations = iterations;
		setNumParameter(NUM_ITERATIONS, iterations);
		return iterations;
	}

	/*
	 * Returns the current iteration of the client simulation
	 */
	public int getCurrentIteration() {
		String command = "s.get_iteration()";
		sendCommand(command);
		try {
			return Integer.parseInt(readResponse());
		} catch (NumberFormatException e) {
//			e.printStackTrace();
			return -1;
		}
	}

	public void setCurrentIteration(int newIteration) {
		String command = "s.set_iteration('" + newIteration + "')";
		sendCommand(command);

		readResponse();
	}

	public void setCellSpeciesValue(String cellID, String speciesName,
			double value) {
//		String command = "s.set_cell_species_data('" + cellID + "', '" + speciesName + "', '" + value + "')";
		String command = "s.set_species_data('" + speciesName + "', {" + cellID + " : " + value + "})";
		// TODO implement in server
		sendCommand(command);

		isTrue(readResponse());
	}

	/*
	 * Sets the timestep of the simulation
	 */
	public double setTimestep(double timestep) {
//		this.timestep = timestep;
		setNumParameter(TIMESTEP, timestep);
		return timestep;
	}

	public double getTimestep() {
		sendCommand("s.get_parameter('"+TIMESTEP + "')");
		String result = readResponse();
		double timestep = 0.2;
		try {
			timestep = Double.parseDouble(result);
		} catch (NumberFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return timestep;
	}

	/*
	 * 
	 */
	public void reset() {
		// TODO
	}

	// public void step(int steps) {
	// for (int i = 0; i < steps; i++)
	// step();
	// }

	/*
	 * Performs another iteration
	 */
	public synchronized int step() {
		String command = "s.step()";
		sendCommand(command);

		String response = readResponse();
//		iteration++;
		int simIteration = Integer.parseInt(response);
//		if (simIteration != iteration) {
//			System.out.println("step() failed: returned " + simIteration
//					+ " rather than " + iteration);
//		}
		return simIteration;
	}

	// public int run() {
	// String command = "s.run()";
	// sendCommand(command);
	//
	// for (; iteration < maxIterations; iteration++) {
	// String response = readResponse();
	// int simIteration = Integer.parseInt(response);
	// if (simIteration != iteration + 1) {
	// ico.printErr("run() failed at iteration " + (iteration + 1)
	// + "\b");
	// }
	// }
	// return iteration;
	// }

	public void export() {

	}

	@Override
	public Map<String, Parameter> getModelDefaultParameters(String modelName)
			throws Exception {
		String command = "s.get_model_default_parameters('" + modelName + "')";
		sendCommand(command);

		return parseParamDictionary(readResponse());
	}

	@Override
	public Map<String, Double> getSpeciesData(String speciesName) {
		String command = "s.get_species_data('" + speciesName + "')";
		sendCommand(command);

		try {
			return parseStringDoubleDictionary(readResponse());
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public String getGeometry() {
		String command = "s.get_geometry()";
		sendCommand(command);
		
		return readResponse();
	}

	@Override
	public Set<Species> getLoadedSpecies() {
		String command = "s.get_loaded_species_names()";
		sendCommand(command);
		Set<Species> result = new HashSet<Species>();

		try {
			Map<String, String> map = parseStringDictionary(readResponse());
			if (map.containsKey("auxin_wall"))
				map.remove("auxin_wall");
			for (String name : map.keySet()) {
				Species sp = new Species(name, getSpeciesPublicName(name), Species.Type.fromString(map.get(name)));
				result.add(sp);
			}			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	public String[] getAllModelSpeciesNames(String modelName) {
		String command = "s.get_model_species_names('" + modelName + "')";
		sendCommand(command);

		return parseStringArray(readResponse());
	}

	public String getSpeciesPublicName(String speciesName){
//		s.get_species_annotation(self, species, realname)
		String command = "s.get_species_annotation('" + speciesName + "', 'realname')";
		sendCommand(command);

		return readResponse();
	}
	
	@Override
	public Map<String, Species> getModelSpecies(String modelName,
			Species.Type speciesType) {
		String command = "s.get_model_species_names('" + modelName + "'"
				+ (speciesType != null ? ",'" + speciesType + "'" : "") + ")";
		sendCommand(command);

		Map<String, Species> result = new HashMap<String, Species>();
		try {
			// <species name, species type>
			Map<String, String> map = parseStringDictionary(readResponse());
			for (String speciesName : map.keySet())
				result.put(speciesName,	new Species(speciesName, getSpeciesPublicName(speciesName), Species.Type.fromString(map.get(speciesName))));
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result;
	}

	
	
	@Override
	public String[] getModelCellSpeciesNames(String modelName) {
		String command = "s.get_model_cell_species_names('" + modelName + "')";
		sendCommand(command);

		return parseStringArray(readResponse());
	}

	@Override
	public SimulationModel getSimulationModel(String modelName)
			throws Exception {
		SimulationModel sm = new SimulationModel(modelName);
		Map<String, Species> speciesMap = getModelSpecies(modelName, null);
		for (Species s : speciesMap.values()) {
			sm.addSpecies(s);
		}
		Map<String, Parameter> params = getModelDefaultParameters(modelName);
		sm.setParameters(params);
		return sm;
	}

	@Override
	public boolean geometryChanged() {
		String command = "s.geometry_changed()";
		sendCommand(command);

		return isTrue(readResponse());
	}

	@Override
	public boolean setCellCarrier(String[] groups, boolean[] onOff) {
		String dictValues = buildPythonDictionary(groups, onOff);
		String command = "s.set_carrier_on_celltype(" + dictValues + ")";
		sendCommand(command);

		return isTrue(readResponse());
	}

	@Override
	public Record getRecord() {
		Record r = new Record(null);
		// load geometry
		Geometry g = Geometry.parseXMLString(getGeometry());
		r.setGeometry(g);
		// load iteration number
		r.setIteration(getCurrentIteration());
		// load species records
		Set<Species> speciesSet = getLoadedSpecies();
//		for (Species species : speciesSet) {
//			switch (species.getType()) {
//			case CELL:
//			case WALL:
//			case CELL_WALL:
//				Map<String, Double> cellMap = getSpeciesData(species.getName());
//				r.setRecord(species, cellMap);
//				break;
//			default:
//				System.err.println("ERROR: step(), species name <"
//						+ species.getName() + "> not recognised!");
//			}
//		}
		return r;
	}

	public Record getRecord(Geometry existingGeometry, Species targetSpecies) {
		Record r = new Record(null);
		// load geometry
		if (existingGeometry != null) {
			if (!geometryChanged())
				r.setGeometry(existingGeometry);
			else {
				Geometry g = Geometry.parseXMLString(getGeometry());
				r.setGeometry(g);
			}
		}
		// load iteration number
		r.setIteration(getCurrentIteration());
		// load species records
		Set<Species> speciesSet = null;
		if (targetSpecies != null) {
			speciesSet = new HashSet<Species>();
			speciesSet.add(targetSpecies);
		} else {
			speciesSet = getLoadedSpecies();
		}
		for (Species species : speciesSet) {
//			switch (species.getType()) {
//			case CELL:
//			case WALL:
//			case CELL_WALL:
				Map<String, Double> cellMap = getSpeciesData(species.getName());
				r.setRecord(species, cellMap);
//				break;
//			default:
//				System.err.println("ERROR: step(), species name <"
//						+ species.getName() + "> not recognised!");
//			}
		}

		return r;
	}

	@Override
	public void setSpeciesData(String speciesName, Map<String, Double> data) {
		// TODO Auto-generated method stub		
		String dataDictionary = buildStringDoublePythonDictionary(data);
		String command = "s.set_species_data('" + speciesName + "', " + dataDictionary + ")";
		
		sendCommand(command);
		
		isTrue(readResponse());
	}

	@Override
	public String[] getAvailableTissueNames() {
		if (socket == null)
			return null;
		sendCommand("s.list_available_tissuefiles()");

		return parseStringArray(readResponse());
	}
}
