package simserver;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import rootgui.model.Geometry;
import rootgui.model.Parameter;
import rootgui.model.objects.SimulationModel;
import rootgui.model.objects.Species;
import rootgui.model.recordstore.Record;
import rootgui.view.IConsoleOutput;
import rootgui.view.SystemOutputAdapter;

/*
 * 
 */
public class SimServerHTTPClient extends AbsSimServerClient {	
	// Communication
	
	
	/*
	protected InputStream in;
	protected OutputStream out;
	protected BufferedReader br;
	protected PrintWriter pw;
	*/

	private boolean connected = false;
	protected String base_url;
	protected String sim_url = null;
	protected int port;

	
	//int timeoutMs = 1500;
	private HttpURLConnection connection;
	
	private IConsoleOutput ico;

	/*
	 * ID corresponding to the VisualiserTab for which this connection is
	 * running.
	 */
	private int id = 0;
	private int INITIAL_RETRY_TIME = 1;
	private int MAX_RETRY_TIME = 16;


	public SimServerHTTPClient(int id, String base_url, int port, IConsoleOutput ico)
			 {
		this.base_url = base_url;
		this.port = port;
		this.id = id;
		
		//System.setProperty("java.net.useSystemProxies", "true");

		
		
		
		try {
			connect();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if (ico == null)
			this.ico = new SystemOutputAdapter();
		else
			this.ico = ico;
	}

	public void connect() throws IOException  {
		connection = (HttpURLConnection) new URL(base_url+"/auxin/sims/").openConnection();
		connection.setDoOutput(true);
		connection.setRequestMethod("POST");
		//br = new BufferedReader(new InputStreamReader(in));
		//pw = new PrintWriter(new OutputStreamWriter(out), true);

		connection.connect();
		sim_url = base_url + connection.getHeaderFields().get("Location").get(0);
		System.out.println("sim_url: " + sim_url);
		connected = true;
	}
	
	@Override
	public void reconnect() throws IOException  {
		
		if (sim_url==null || connected==false)
		{
			connect();
			return;
		}

	}

	public String getHostname() {
		return this.base_url;
	}


	/*
	 * Irreversibly disconnects the client from the Server.
	 */
	@Override
	public synchronized void disconnect() {
		connection.disconnect();
		connected = false;
	}



	public static String bytesToHex(byte[] bytes) {
        StringBuffer result = new StringBuffer();
        for (byte byt : bytes) result.append(Integer.toString((byt & 0xff) + 0x100, 16).substring(1));
        return result.toString();
    }
	/*
	 * 
	 * @see simserver.AbsSimServerClient#isConnected()
	 */
	public boolean isConnected() {
		return connected;
	}

	private String sendCommand(String command) {
		return sendCommand(command, false);
	}

	/*
	 * 
	 */
	private  synchronized  String sendCommand(String command, boolean printOutput) {
		int retry_time = INITIAL_RETRY_TIME;
		if(!connected)
			return null;
		String response = null;
		while(response == null)
		{
			try {
				connection = (HttpURLConnection) new URL(sim_url).openConnection();
				connection.setDoOutput(true);
				connection.setRequestMethod("POST");
				OutputStream output = connection.getOutputStream();	
				output.write(command.getBytes());
				//connection.connect();
				
				int code = connection.getResponseCode();
				
				if (printOutput)
					System.out.println(sim_url+ " command: " + command);
				if (printOutput)
					System.out.println("code: " + code);
				
				if(code<200 || code>=300)
				{
					if(code>=502 || code<=504)
					{
						if(retry_time>MAX_RETRY_TIME)
						{
							connected=false;
							return null;
						}
						Thread.sleep(retry_time);
						retry_time*=2;
						System.err.println("Retry");
						continue;
					}
					else
					{
						System.out.println("HTTP Error code: "+code);
						connected=false;
						return null;
					}
				}
				else
				{
					retry_time = INITIAL_RETRY_TIME;
					break;
				}
			}  catch (IOException e) {
				if(retry_time>MAX_RETRY_TIME)
				{
					connected=false;
					return null;
				}
				try {
					Thread.sleep(retry_time);
				} catch (InterruptedException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
				retry_time*=2;
				System.err.println("IO Exception");
				continue;
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 
			
		}
					
		InputStream input;
		try {
			input = connection.getInputStream();

			BufferedReader br = new BufferedReader(new InputStreamReader(input));
			response = br.readLine();		
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (printOutput) 
		{
			if(response != null)
				System.out.println("response: " + response.substring(0,Math.min(response.length(), 200)));
			else
				System.out.println("response: null");
		}
		
		return response;	
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simserver.AbsSimServerClient#setTissueFile(java.lang.String)
	 */
	public boolean setTissueFile(String tissueFile)  {
		String r = sendCommand("s.set_tissue_name('" + tissueFile + "')");

		if (!tissueFile.equals(r)) {
			ico.printErr(id, "SimServerClient: setTissueFile failed\n");
			return false;
		}
		return true;
	}

	/*
	 * 
	 * @see
	 * simserver.AbsSimServerClient#addModel(rootgui.model.objects.SimulationModel
	 * )
	 */
	public boolean addModel(SimulationModel sm) {
		String r = sendCommand("s.add_model_by_name('" + sm.getName() + "')");

		if (!isTrue(r)) {
			ico.printErr(id,
					"SimServerClient.addModel(): failed at add_model_by_name() stage\n");// ,
																							// IConsoleOutput.DEBUG
			return false;
		} else {
			// upload all parameters
			for (String parName : sm.getParameters().keySet()) {
				setParameter(sm.getName(), parName,
						sm.getParameters().get(parName).getValue());
			}
		}
		return true;
	}

	@Override
	public boolean addEditedModel(SimulationModel editedModel,
			SimulationModel defaultModel) {
		String r = sendCommand("s.add_model_by_name('" + editedModel.getName() + "')");

		if (!isTrue(r)) {
			ico.printErr(id,
					"SimServerClient.addModel(): failed at add_model_by_name() stage\n");
			return false;
		} else {
			// add only modified parameters
			for (String parName : editedModel.getParameters().keySet()) {
				if (!editedModel.getParameterValue(parName).equals(defaultModel.getParameterValue(parName)))
					setParameter(editedModel.getName(), parName, editedModel.getParameters().get(parName).getValue());
			}
		}

		return true;
	}

	public void addModelByName(String modelName) {
		String r = sendCommand("s.add_model_by_name('" + modelName + "')");

		if (!isTrue(r)) {
			ico.printErr(id, "SimServerClient.addModelByName(): failed\n");
		}
	}

	public void setSBMLModel(String fileName) {
		String r = sendCommand("s.add_SBML_model('" + fileName + "')");

		if (!isTrue(r)) {
			ico.printErr(id, "SimServerClient.setSBMLModel(): failed\n");
		}
	}

	public void unsetModel(String modelName) {
		sendCommand("s.remove_model_by_name('" + modelName + "')");
	}

	@Override
	public String[] getAvailableModelsNames() throws IOException {
		
		String r = sendCommand("s.list_available_models()");

		return parseStringArray(r);
	}

	@Override
	public boolean initTissue() {
		String r = sendCommand("s.initialize_tissue()");

		if (!isTrue(r)) {
			System.out.println("Init tissue failed");
			return false;
		}
		return true;
	}

	@Override
	public boolean initSimulation() {
		String command = "s.init_simulation()";
		String r = sendCommand(command);

		return isTrue(r);
	}

	/*
	 * Returns the species loaded in the simulation
	 */
	public String[] listSpecies() throws IOException {
		String command = "s.list_species()";
		String r = sendCommand(command);

		return parseStringArray(r);
	}
	
	public boolean setGrowth(boolean onOff)  {
		// TODO have it turning off?
//		System.err.println("Growth about to be set");
		String command = "s.add_module('growth')";
		sendCommand(command);

		return true;
	}
	
	public boolean setParameter(String modelName, String parameter, String value) {
		if (modelName != null && modelName.length() > 0)
			parameter = modelName + "." + parameter;
		String r = sendCommand("s.set_parameter('" + parameter + "', '" + value + "')");

		// ico.printlnStd(s);
		return true;
	}

	public String getParameter(String modelName, String parameter) {
		if (modelName != null && modelName.length() > 0)
			parameter = modelName + "." + parameter;
		String r = sendCommand("s.get_parameter('" + parameter + "')");

		return r;
	}

	public float getNumParameter(String modelName, String parameter) {
		if (modelName != null && modelName.length() > 0)
			parameter = modelName + "." + parameter;
		String r = sendCommand("s.get_parameter('" + parameter + "')");

		return Float.parseFloat(r);
	}

	/*
	 * Sets a numerical parameter
	 */
	public void setNumParameter(String parameter, double value) {
		String s = sendCommand("s.set_parameter('" + parameter + "', " + value + ")");

		 System.out.println(s);
	}

	public int setIterations(int iterations) {
//		maxIterations = iterations;
		setNumParameter(NUM_ITERATIONS, iterations);
		return iterations;
	}

	/*
	 * Returns the current iteration of the client simulation
	 */
	public int getCurrentIteration() {
		String command = "s.get_iteration()";
		String r = sendCommand(command);
		try {
			return Integer.parseInt(r);
		} catch (NumberFormatException e) {
//			e.printStackTrace();
			return -1;
		}
	}

	public void setCurrentIteration(int newIteration) {
		String command = "s.set_iteration('" + newIteration + "')";
		sendCommand(command);
	}

	public void setCellSpeciesValue(String cellID, String speciesName,
			double value) {
//		String command = "s.set_cell_species_data('" + cellID + "', '" + speciesName + "', '" + value + "')";
		String command = "s.set_species_data('" + speciesName + "', {" + cellID + " : " + value + "})";
		// TODO implement in server
		String r = sendCommand(command);

		isTrue(r);
	}

	/*
	 * Sets the timestep of the simulation
	 */
	public double setTimestep(double timestep) {
//		this.timestep = timestep;
		setNumParameter(TIMESTEP, timestep);
		return timestep;
	}

	public double getTimestep() {
		String result = sendCommand("s.get_parameter('"+TIMESTEP + "')");
		double timestep = 0.2;
		try {
			timestep = Double.parseDouble(result);
		} catch (NumberFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return timestep;
	}

	/*
	 * 
	 */
	public void reset() {
		String command = "s.reset_simulation()";
		String r = sendCommand(command);

	}

	// public void step(int steps) {
	// for (int i = 0; i < steps; i++)
	// step();
	// }

	/*
	 * Performs another iteration
	 */
	public synchronized int step() {
		String command = "s.step()";
		String response = sendCommand(command);

//		iteration++;
		int simIteration = Integer.parseInt(response);
//		if (simIteration != iteration) {
//			System.out.println("step() failed: returned " + simIteration
//					+ " rather than " + iteration);
//		}
		return simIteration;
	}

	// public int run() {
	// String command = "s.run()";
	// sendCommand(command);
	//
	// for (; iteration < maxIterations; iteration++) {
	// String response = readResponse();
	// int simIteration = Integer.parseInt(response);
	// if (simIteration != iteration + 1) {
	// ico.printErr("run() failed at iteration " + (iteration + 1)
	// + "\b");
	// }
	// }
	// return iteration;
	// }

	public void export() {

	}

	@Override
	public Map<String, Parameter> getModelDefaultParameters(String modelName)
			throws Exception {
		String command = "s.get_model_default_parameters('" + modelName + "')";
		String r = sendCommand(command);

		return parseParamDictionary(r);
	}

	@Override
	public Map<String, Double> getSpeciesData(String speciesName) {
		String command = "s.get_species_data('" + speciesName + "')";
		String r = sendCommand(command);

		try {
			return parseStringDoubleDictionary(r);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public String getGeometry() {
		String command = "s.get_geometry()";
		return sendCommand(command);
		
	}

	@Override
	public Set<Species> getLoadedSpecies() {
		String command = "s.get_loaded_species_names()";
		String r = sendCommand(command);
		Set<Species> result = new HashSet<Species>();

		try {
			Map<String, String> map = parseStringDictionary(r);
			if (map.containsKey("auxin_wall"))
				map.remove("auxin_wall");
			for (String name : map.keySet()) {
				Species sp = new Species(name, getSpeciesPublicName(name), Species.Type.fromString(map.get(name)));
				result.add(sp);
			}			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	public String[] getAllModelSpeciesNames(String modelName) {
		String command = "s.get_model_species_names('" + modelName + "')";
		String r = sendCommand(command);

		return parseStringArray(r);
	}

	public String getSpeciesPublicName(String speciesName){
//		s.get_species_annotation(self, species, realname)
		String command = "s.get_species_annotation('" + speciesName + "', 'realname')";
		String r = sendCommand(command);

		return r;
	}
	
	@Override
	public Map<String, Species> getModelSpecies(String modelName,
			Species.Type speciesType) {
		String command = "s.get_model_species_names('" + modelName + "'"
				+ (speciesType != null ? ",'" + speciesType + "'" : "") + ")";
		String r = sendCommand(command);

		Map<String, Species> result = new HashMap<String, Species>();
		try {
			// <species name, species type>
			Map<String, String> map = parseStringDictionary(r);
			for (String speciesName : map.keySet())
				result.put(speciesName,	new Species(speciesName, getSpeciesPublicName(speciesName), Species.Type.fromString(map.get(speciesName))));
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result;
	}

	
	
	@Override
	public String[] getModelCellSpeciesNames(String modelName) {
		String command = "s.get_model_cell_species_names('" + modelName + "')";
		String r = sendCommand(command);

		return parseStringArray(r);
	}

	@Override
	public SimulationModel getSimulationModel(String modelName)
			throws Exception {
		SimulationModel sm = new SimulationModel(modelName);
		Map<String, Species> speciesMap = getModelSpecies(modelName, null);
		for (Species s : speciesMap.values()) {
			sm.addSpecies(s);
		}
		Map<String, Parameter> params = getModelDefaultParameters(modelName);
		sm.setParameters(params);
		return sm;
	}

	@Override
	public boolean geometryChanged() {
		String command = "s.geometry_changed()";
		String r = sendCommand(command);

		return isTrue(r);
	}

	@Override
	public boolean setCellCarrier(String[] groups, boolean[] onOff) {
		String dictValues = buildPythonDictionary(groups, onOff);
		String command = "s.set_carrier_on_celltype(" + dictValues + ")";
		String r = sendCommand(command);

		return isTrue(r);
	}

	@Override
	public Record getRecord() {
		Record r = new Record(null);
		// load geometry
		Geometry g = Geometry.parseXMLString(getGeometry());
		r.setGeometry(g);
		// load iteration number
		r.setIteration(getCurrentIteration());
		// load species records
		Set<Species> speciesSet = getLoadedSpecies();
//		for (Species species : speciesSet) {
//			switch (species.getType()) {
//			case CELL:
//			case WALL:
//			case CELL_WALL:
//				Map<String, Double> cellMap = getSpeciesData(species.getName());
//				r.setRecord(species, cellMap);
//				break;
//			default:
//				System.err.println("ERROR: step(), species name <"
//						+ species.getName() + "> not recognised!");
//			}
//		}
		return r;
	}

	public Record getRecord(Geometry existingGeometry, Species targetSpecies) {
		Record r = new Record(null);
		// load geometry
		if (existingGeometry != null) {
			if (!geometryChanged())
				r.setGeometry(existingGeometry);
			else {
				Geometry g = Geometry.parseXMLString(getGeometry());
				r.setGeometry(g);
			}
		}
		// load iteration number
		r.setIteration(getCurrentIteration());
		// load species records
		Set<Species> speciesSet = null;
		if (targetSpecies != null) {
			speciesSet = new HashSet<Species>();
			speciesSet.add(targetSpecies);
		} else {
			speciesSet = getLoadedSpecies();
		}
		for (Species species : speciesSet) {
//			switch (species.getType()) {
//			case CELL:
//			case WALL:
//			case CELL_WALL:
				Map<String, Double> cellMap = getSpeciesData(species.getName());
				r.setRecord(species, cellMap);
//				break;
//			default:
//				System.err.println("ERROR: step(), species name <"
//						+ species.getName() + "> not recognised!");
//			}
		}

		return r;
	}

	@Override
	public void setSpeciesData(String speciesName, Map<String, Double> data) {
		// TODO Auto-generated method stub		
		String dataDictionary = buildStringDoublePythonDictionary(data);
		String command = "s.set_species_data('" + speciesName + "', " + dataDictionary + ")";
		
		String r = sendCommand(command);
		
		isTrue(r);
	}

	@Override
	public String[] getAvailableTissueNames() {
		
		String r = sendCommand("s.list_available_tissuefiles()");

		return parseStringArray(r);
	}
}
