package visualiser;

import javax.swing.JComponent;

import rootgui.controller.ISimulationControl;
import rootgui.controller.ITabSimulationControl;
import rootgui.model.Geometry;
import rootgui.model.IHistDataProvider;
import rootgui.model.ITabHistDataProvider;
import rootgui.model.objects.Cell;
import rootgui.model.objects.Species;
import rootgui.model.recordstore.IRecordStore;
import rootgui.model.recordstore.Record;
import rootgui.model.recordstore.SpeciesRecord;
import rootgui.view.IConsoleOutput;
import visualiser.viewpanel.controller.ViewPanelActionListener;
import visualiser.viewpanel.view.tabs.VisualiserTab;

/*
 * Interface for a component providing visualisation services to the main frame.
 */
public interface IVisualiser extends ViewPanelActionListener {

	/*
	 * Makes the visualiser listen to ViewPanelActions
	 */
	public void addViewPanelActionListener(ViewPanelActionListener l);

	/*
	 * 
	 */
	public void removeViewPanelActionListener(ViewPanelActionListener l);

	public void addVisualiserActionListener(VisualiserActionListener l);
	
	public void removeVisualiserActionListener(VisualiserActionListener l);
	
	/*
	 * Returns the JComponent representing an IVisualiser instance
	 */
	public JComponent getRenderPanel();

	/*
	 * 
	 */
	// public int setRenderedIteration(int iteration);

	/*
	 * 
	 */
	// public int getRenderedIteration();

	/*
	 * Specifies the IConsoleOutput object which will render the IVisualiser's
	 * output
	 */
	public void setConsoleOutput(IConsoleOutput ico);

	/*
	 * Returns the IConsoleOutput object to which the IVisualiser was directing
	 * its output
	 */
	public IConsoleOutput getConsoleOutput();

	/*
	 * Makes available another Record of simulation data, for the currently
	 * active simulation
	 */
	public void setRecord(int simID, Record record, boolean initial);

	/*
	 * Makes available the Geometry, for the currently active simulation
	 */
	// public void setGeometry(Geometry geometry);

	/*
	 * Sets the figure of the selected VisualiserTab
	 */
	void setFigure(int simID, Figure f);

	void setLoadedSpecies(int simID, Species[] species);
	/*
	 * 
	 */
	public Figure getFigure(int simID);

	/*
	 * 
	 */
//	public void setRunEnabled(int simID, boolean enabled);

	public void setMaxIterations(int simID, int iteration);

	/*
	 * Returns the IHistDataProvider instance of a IVisualiser object
	 */
	public ITabHistDataProvider getTabHistDataProvider();

	/*
	 * Returns the ISimulationControl instance of a IVisualiser object
	 */
	public ITabSimulationControl getTabSimulationControl();
	
	// public int getCurrentRecordIteration();

	public void fireRecordUpdated(int simID, int iteration);
	
	/*
	 * 
	 */
	public void setRenderedIteration(int simID, int iteration);

	/* TABS */
	/*
	 * 
	 */
	public void addVisualiserTab(Figure f);

	/*
	 * 
	 */
	public void setSelectedTabIndex(int index);
	
	
	
	/*
	 * 
	 */
	public int getSelectedTabIndex();

	public int getSelectedSimID();
	
	/*
	 * Removes a tab
	 */
	public void removeTabByIndex(int tabIndex);
	
	/*
	 * Resets the view and model of a tab
	 */
	public void resetTab(int simID);
	
	/*
	 * Resets the view and model of the selected tab
	 */
	public void resetSelectedTab();	

	public VisualiserTab getActiveVisualiserTab();
}
