package visualiser;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTabbedPane;

import rootgui.controller.ITabSimulationControl;
import rootgui.model.ITabHistDataProvider;
import rootgui.model.objects.Cell;
import rootgui.model.objects.Species;
import rootgui.model.recordstore.Record;
import rootgui.view.IConsoleOutput;
import rootgui.view.SpeciesSelectionDialog;
import visualiser.viewpanel.controller.ViewPanelActionListener;
import visualiser.viewpanel.model.AbstractModel;
import visualiser.viewpanel.view.tabs.ButtonTabComponent;
import visualiser.viewpanel.view.tabs.MyTabbedPane;
import visualiser.viewpanel.view.tabs.VisualiserTab;

public class Visualiser implements IVisualiser {
	/*
	 * 
	 */
	AbstractModel model;

	/*
	 * Main container for the render panels
	 */
	private MyTabbedPane tabbedPane = new MyTabbedPane();

	private Map<Integer, VisualiserTab> visTabs = new HashMap<Integer, VisualiserTab>();

	JPanel mainPanel = new JPanel();

	private static int simCount = 1;

	/*
	 * Control bar, containing start/stop/backward/forward/zoom in/zoom out
	 * buttons
	 */
	// ControlBar icb;
	/*
	 * Visualiser controller
	 */
	// IController controller;
	/*
	 * Initial VisualiserTab (index 0)
	 */
	VisualiserTab firstTab;

	/*
	 * Console output wrapper
	 */
	IConsoleOutput cOut;

	/*
	 * Reference to the IHistDataProvider object
	 */
	ITabHistDataProvider dp;

	/*
	 * Reference to the ISimulationControl object
	 */
	ITabSimulationControl isc;

	LinkedList<ViewPanelActionListener> listeners;
	LinkedList<VisualiserActionListener> vaListeners;
	/*
	 * Main constructor, creating a visualiser with one tabbed panel.
	 * 
	 * @param IHistDataProvider dp: historical data provider object, giving
	 * access to the repository of geometries and species records
	 * 
	 * @param ISimulationControl isc: object giving access to the simulation
	 * control
	 */
	public Visualiser(ITabHistDataProvider dp, ITabSimulationControl isc) {
		this.listeners = new LinkedList<ViewPanelActionListener>();
		vaListeners = new LinkedList<VisualiserActionListener>();
		this.dp = dp;
		this.isc = isc;

		mainPanel.setLayout(new BorderLayout());
		mainPanel.add(tabbedPane, BorderLayout.CENTER);
		// Set the layout policy to allow suit the scroll tab
		tabbedPane.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
		installMouseListenerWrapper();
//		tabbedPane.addChangeListener(new ChangeListener() {
//			
//			@Override
//			public void stateChanged(ChangeEvent e) {
//				// TODO Auto-generated method stub
//				
//			}
//		});
//		tabbedPane.addMouseListener(new MouseListener() {
//
//			@Override
//			public void mouseReleased(MouseEvent e) {
//				// TODO Auto-generated method stub
//
//			}
//
//			@Override
//			public void mousePressed(MouseEvent e) {
//				// TODO Auto-generated method stub
//			}
//
//			@Override
//			public void mouseExited(MouseEvent e) {
//				// TODO Auto-generated method stub
//
//			}
//
//			@Override
//			public void mouseEntered(MouseEvent e) {
//				// TODO Auto-generated method stub
//
//			}
//
////			@Override
////			public void mouseClicked(MouseEvent e) {
////				if (SwingUtilities.isLeftMouseButton(e)){
//////				System.err.println("Visualiser.mouseClicked: " + tabbedPane.getSelectedIndex());
////				int index = tabbedPane.getSelectedIndex();				
////				if ((tabbedPane.getSelectedIndex() != tabbedPane.getPreviouslySelectedIndex()) && (tabbedPane.getSelectedComponent().getName() != null)) 
//////				 System.out.printf("Old tab index: %d, new tab index: %d, size: %d\n", oldTabIndex, newIndex, tabbedPane.getTabCount());
//////				if (newIndex != oldTabIndex && (newIndex < tabbedPane.getTabCount() - 1) && newIndex >= 0) {
//////					setSelectedTabIndex(tabbedPane.getSelectedIndex());
//////					oldTabIndex = newIndex;
////					for (VisualiserActionListener l : vaListeners) {
////						l.fireTabSelected(index, ((VisualiserTab)tabbedPane.getSelectedComponent()).getSimID());
////					}
//////				}
////				}
//				
//				if (SwingUtilities.isRightMouseButton(e)){
//					
//					System.err.println("Visualiser.mouseClicked: " + tabbedPane.getSelectedIndex());
//////					int index = tabbedPane.getSelectedIndex();				
////					if ((tabbedPane.getSelectedIndex() != tabbedPane.getPreviouslySelectedIndex()) && (tabbedPane.getSelectedComponent().getName() != null)) 
////						for (VisualiserActionListener l : vaListeners) {
////							l.fireTabSelected(index, ((VisualiserTab)tabbedPane.getSelectedComponent()).getSimID());
////						}
////					}
//					}
//				
//			}
//		});
	}

	/*
	 * Creates a new VisualiserTab
	 */
	public void addVisualiserTab(final Figure f) {
		final Visualiser thisVisualiser = this;
		
//		SwingUtilities.invokeLater(new Runnable() {
//			@Override
//			public void run() {
				VisualiserTab tb = new VisualiserTab(thisVisualiser, f, simCount);
				int tabIndex = visTabs.size();
				// tb.setRecord(dp.getLastAvailableRecord(tabIndex), false);
				if (visTabs.size() == 0)
					firstTab = tb;
				visTabs.put(simCount, tb);

				/** Tabs */
				// Add the panel to the tab
				tabbedPane.add(tb, "Simulation " + simCount);
				tabbedPane.setTabComponentAt(tabbedPane.getTabCount() - 1, new ButtonTabComponent(tabbedPane, VisualiserTab.closeIcon, "Close this tab", thisVisualiser));
				tabbedPane.add(new JPanel(), null);
				tabbedPane.setEnabledAt(tabbedPane.getTabCount() - 1,false);
				tabbedPane.setTabComponentAt(tabbedPane.getTabCount() - 1, new ButtonTabComponent(tabbedPane, VisualiserTab.addIcon, "Create new tab", thisVisualiser));

				for (VisualiserActionListener l : vaListeners)
					l.fireTabAdded(tabIndex, simCount);
				tabbedPane.setSelectedIndex(tabIndex);
//				 tabbedPane.setEnabledAt(tabbedPane.getTabCount() - 1, false);
//				if (tabIndex != 0)
//					setSelectedTabIndex(tabIndex);
//				if (tabIndex > 0)					
				simCount++;
//			}
//		});		
	}

	// private int oldSelectedTabIndex = 0;

	@Override
	public void removeTabByIndex(final int index) {
		new Thread(new Runnable() {
			
			@Override
			public void run() {
				int simID = ((VisualiserTab)tabbedPane.getComponentAt(index)).getSimID();
				for (VisualiserActionListener l : vaListeners)
					l.fireTabRemoved(index, simID);
				visTabs.remove(simID);
				tabbedPane.removeTabAt(index);
				setSelectedTabIndex(index - 1);				
			}
		}).start();
	}

	@Override
	public int getSelectedTabIndex() {
		return tabbedPane.getSelectedIndex();
	}
	
	@Override
	public int getSelectedSimID(){
			
			return ((VisualiserTab) tabbedPane.getSelectedComponent()).getSimID();
	}

	@Override
	public void setSelectedTabIndex(int newIndex) {
		if (newIndex < 0)
			newIndex = 0;
		else if (newIndex >= visTabs.size())
			newIndex = visTabs.size() - 1;
//		if (tabbedPane.getPreviouslySelectedIndex() == newIndex) {
		tabbedPane.setSelectedIndex(newIndex);
		// if (oldSelectedTabIndex != newIndex) {
		 for (VisualiserActionListener l : vaListeners)
			 l.fireTabSelected(tabbedPane.getSelectedIndex(), ((VisualiserTab)tabbedPane.getSelectedComponent()).getSimID());
//		}
		// oldSelectedTabIndex = newIndex;
		// }
	}

	public void setFigureToActiveTab(Figure f) {
		VisualiserTab tab = getActiveVisualiserTab();
		tab.setFigure(f);
		Record record = tab.getRecord();
		tab.setRecord(record, true, false);
	}

	// @Override
	public void reset() {
	}

	public javax.swing.JComponent getRenderPanel() {
		return mainPanel;
	}

	public void setConsoleOutput(IConsoleOutput co) {
		cOut = co;
	}

	@Override
	public IConsoleOutput getConsoleOutput() {
		return cOut;
	}

	@Override
	public void setRecord(int simID, Record record, boolean initial) {
		visTabs.get(simID).setRecord(record, true, initial);
	}

	@Override
	public void setFigure(int simID, Figure figure) {
		visTabs.get(simID).setFigure(figure);
	}

	@Override
	public void setLoadedSpecies(int simID, Species[] species) {
		visTabs.get(simID).setLoadedSpecies(species);
	}

//	@Override
//	public void setFitZoomRatio(int simID){
//		visTabs.get(simID).set
//	}
	
	private void installMouseListenerWrapper() {
		final JPopupMenu tabPopup = new JPopupMenu();
		final JMenuItem menuSpecies = new JMenuItem("Select species");
		menuSpecies.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				Species [] loadedSpecies = getTabHistDataProvider().getLoadedSpeciesArray(getSelectedSimID());				
				SpeciesSelectionDialog.showDialog(getRenderPanel(), null, "Species: ","Select species",  loadedSpecies);
				if (SpeciesSelectionDialog.getButtonAction() == SpeciesSelectionDialog.OK
						&& SpeciesSelectionDialog.getSpecies().length > 0) {
					Figure f = new Figure(SpeciesSelectionDialog.getSpecies(),
							SpeciesSelectionDialog.getMinRanges(),
							SpeciesSelectionDialog.getMaxRanges(),
							SpeciesSelectionDialog.getSelectedRows());
					f.setNoColumns(SpeciesSelectionDialog.getNoColumns());
					
					setFigureToActiveTab(f);							
				}
			}
		});
		tabPopup.add(menuSpecies);
//		
		JMenuItem menuClose = new JMenuItem("Close");
		menuClose.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (tabbedPane.getTabCount() > 2) {  	
				removeTabByIndex(getSelectedTabIndex());
				}
			}
		});
		tabPopup.add(menuClose);
		
//		MouseListener handler = findUIMouseListener();
//		if (handler == null)
//			handler = new MouseListener() {
//
//				@Override
//				public void mouseClicked(MouseEvent e) {
//					// TODO Auto-generated method stub
//
//				}
//
//				@Override
//				public void mousePressed(MouseEvent e) {
//					// TODO Auto-generated method stub
//
//				}
//
//				@Override
//				public void mouseReleased(MouseEvent e) {
//					// TODO Auto-generated method stub
//
//				}
//
//				@Override
//				public void mouseEntered(MouseEvent e) {
//					// TODO Auto-generated method stub
//
//				}
//
//				@Override
//				public void mouseExited(MouseEvent e) {
//					// TODO Auto-generated method stub
//
//				}
//
//			};
//		if (handler != null) {
//
//			tabbedPane.removeMouseListener(handler);
//			tabbedPane.addMouseListener(new MouseListenerWrapper(handler, this));
			tabbedPane.addMouseListener(new MouseAdapter() {
			
				public void mousePressed(MouseEvent e) {
					maybeShowPopup(e);
				}

				public void mouseReleased(MouseEvent e) {
					maybeShowPopup(e);
				}

				public void maybeShowPopup(MouseEvent e) {
					if (e.isPopupTrigger()) {
						Species[] loadedSpecies=getTabHistDataProvider().getLoadedSpeciesArray(getSelectedSimID());
						menuSpecies.setEnabled(loadedSpecies != null && loadedSpecies.length > 0);							
						tabPopup.show(e.getComponent(), e.getX(), e.getY());
					} else 	{			
						int index = tabbedPane.getSelectedIndex();
						//tabbedPane.getSelectedIndex() != tabbedPane.getPreviouslySelectedIndex()) && 
						if ((index < tabbedPane.getTabCount() -1)) 
							for (VisualiserActionListener l : vaListeners) {
								l.fireTabSelected(index, ((VisualiserTab)tabbedPane.getSelectedComponent()).getSimID());
							}
									
					}
				}
			});
//			
//		}
	}

//	private MouseListener findUIMouseListener() {
//		MouseListener[] listeners = tabbedPane.getMouseListeners();
//		for (MouseListener l : listeners) {
//			if (l.getClass().getName().contains("$Handler")) {
//				return l;
//			}
//		}
//		return null;
//	}

	public VisualiserTab getActiveVisualiserTab() {
		VisualiserTab activeTab = (VisualiserTab) tabbedPane.getSelectedComponent();
		return activeTab;
	}

	@Override
	public void setMaxIterations(int simID, int maxIterations) {
		visTabs.get(simID).setMaxIterations(maxIterations);
	}

	@Override
	public ITabHistDataProvider getTabHistDataProvider() {
		return dp;
	}

	@Override
	public ITabSimulationControl getTabSimulationControl() {
		return isc;
	}

	@Override
	public Figure getFigure(int simID) {
		// if (visTabs != null && visTabs.size() > 0)
		return visTabs.get(simID).getFigure();
		// return null;
	}

	@Override
	public void addVisualiserActionListener(VisualiserActionListener l) {
		vaListeners.add(l);
	}

	@Override
	public void removeVisualiserActionListener(VisualiserActionListener l) {
		vaListeners.remove(l);
	}

	@Override
	public void addViewPanelActionListener(ViewPanelActionListener l) {
		listeners.add(l);
	}

	@Override
	public void removeViewPanelActionListener(ViewPanelActionListener l) {
		listeners.remove(l);
	}

	public void fireCellSelected(final int simID, Record record, Cell selectedCell) {
		for (ViewPanelActionListener l : listeners) {
			l.fireCellSelected(simID, record, selectedCell);
		}
	}
	
	public void fireCellUnselected(final int simID) {
		for (ViewPanelActionListener l : listeners) {
			l.fireCellUnselected(simID);
		}
	}

	@Override
	public void fireRecordUpdated(int simID, int iteration) {
		if (visTabs.get(simID).getRecord().getIteration() == iteration)
			visTabs.get(simID).setRenderedIteration(iteration);
	}

	/*
	 * 
	 * @see visualiser.IVisualiser#setRenderedIteration(int, int)
	 */
	@Override
	public void setRenderedIteration(int tabIndex, int iteration) {
		visTabs.get(tabIndex).setRenderedIteration(iteration);
	}

	@Override
	public void resetTab(int simID) {
//		if (index >= 0 && index < visTabs.size())
			visTabs.get(simID).reset();
	}

	@Override
	public void resetSelectedTab() {
		resetTab(tabbedPane.getSelectedIndex());
	}
}
