package visualiser.viewpanel.controller;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;

import rootgui.model.Geometry;
import rootgui.model.IHistDataProvider;
import rootgui.model.ITabHistDataProvider;
import rootgui.model.objects.Cell;
import rootgui.model.recordstore.Record;
import visualiser.Figure;
import visualiser.viewpanel.model.AbstractModel;
//import visualiser.model.VisualiserModelActionListener;
import visualiser.viewpanel.view.CellDrawingPanel;
import visualiser.viewpanel.view.CellView;
import visualiser.viewpanel.view.VisualiserPanel;
import visualiser.viewpanel.view.tabs.VisualiserTab;

public class CDPController extends MouseAdapter implements MouseListener,
		ICDPController, ViewPanelActionListener {
	private ITabHistDataProvider dp;
	private CellDrawingPanel view;
	private Record currentRecord;
	// private Figure figure;

	Collection<CellView> selectedCellViews;
	private Point mousePressedPoint = null;

	LinkedList<ViewPanelActionListener> listeners;
	VisualiserTab tab;

	public CDPController(VisualiserTab parent, ITabHistDataProvider dp,
			CellDrawingPanel view) {
		this.listeners = new LinkedList<ViewPanelActionListener>();
		this.dp = dp;
		this.tab = parent;
		this.view = view;
		//
		selectedCellViews = new ArrayList<CellView>();

		view.addMouseWheelListener(this);
		view.addMouseListener(this);
		view.addMouseMotionListener(this);
		view.setController(this);
	}

	public ITabHistDataProvider getModel() {
		return dp;
	}

	@Override
	public void setRenderPanel(CellDrawingPanel view) {
		// TODO this.view = view;
		view.setController(this);
	}

	@Override
	public Collection<Cell> getCells() {
		if (currentRecord != null)
			try {
				return currentRecord.getGeometry().getCells();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		return null;
	}

	@Override
	public Record getRecord() {
		return currentRecord;
	}

	@Override
	public void setRecord(Record r, boolean initial) {
		currentRecord = r;
		view.initCellsView(initial);
		
		if (r == null) {
			selectedCellViews.clear();			
		}
		/*
		 * update information of selected cells in the cell information display panel 
		 */
		for (CellView cv : selectedCellViews) {
			if(cv!=null) {
			tab.fireCellSelected(currentRecord, cv.getCell());
			}
		}
		
		view.repaint();
	}

	@Override
	public void mouseClicked(MouseEvent e) {

		if (currentRecord != null) {

			for (CellView cv : selectedCellViews) {
				if(cv!=null)
				{
					cv.setSelected(false);
				}
			}
			selectedCellViews.clear();

			Point p = e.getPoint();
			CellView cv = view.pointInCell(p);
			if (cv != null) {
				cv.setSelected(true);
				selectedCellViews.add(cv);
				tab.fireCellSelected(currentRecord, cv.getCell());
			} else {
				tab.fireCellUnselected();
			}
			view.repaint();
			if (view.isPressedCloseIcon(p))
				//System.err.println("close!!!");
				if (tab.getFigure().getSelectedSpeciesCount() > 1)
					tab.closeVisualiserPanel(view.getId());
		}
	}

	@Override
	public void mousePressed(MouseEvent e) {
		Point p = e.getPoint();
		mousePressedPoint = p;
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mouseEntered(MouseEvent e) {
		

	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	@Override
	public void mouseDragged(MouseEvent e) {
		if (mousePressedPoint != null) {
			Point p = e.getPoint();
			double tx = p.x - mousePressedPoint.getX();
			double ty = p.y - mousePressedPoint.getY();
			tab.applyTransform(view.getId(), 1, tx, ty, false);

			mousePressedPoint = p;
		}
	}
	
	@Override
	public void mouseMoved(MouseEvent e) {
		Point p=e.getPoint();
		if (view.isPressedCloseIcon(p)) {
			view.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		} else {
			view.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
		super.mouseMoved(e);
	}

	@Override
	public void mouseWheelMoved(MouseWheelEvent e) {
		if (currentRecord != null) {
			Point p = e.getPoint();
			int notches = e.getWheelRotation();
			double sx = (notches > 0) ? 1 - VisualiserPanel.ZOOM_INCREMENT : 1 + VisualiserPanel.ZOOM_INCREMENT;
			double sy = sx;

			double tx = p.x * (1 - sx);
			double ty = p.y * (1 - sy);
//			System.out.println("x: " + p.x + "  y: " + p.y + " // " + sx + " tx: " + tx + " ty: " + ty);
//	        -( p.x / (1 - sx) + originx - p.x / ( (1 - sx) * zoom ) ),
//	        -( p.y / (1 - sx) + originy - p.y / ( (1 - sx) * zoom ) )
			
			tab.applyTransform(view.getId(), sx, tx, ty, false);
			tab.setSpinnerValue();
		}
	}

	@Override
	public void setFigure(Figure f, int index) {
		view.setFigure(f, index);
	}

	public Collection<CellView> getSelectedCellViews() {
		return selectedCellViews;
	}

	@Override
	public void fireCellUnselected(final int simID) {
		selectedCellViews.clear();
		view.repaint();
	}
	
	@Override
	public void fireCellSelected(final int simID, Record record, Cell selectedCell) {
		if (currentRecord != null) {
			if (selectedCellViews.size() > 0)
				for (CellView cv : selectedCellViews)
					cv.setSelected(false);

			selectedCellViews.clear();

			CellView cv = view.getCellViewById(selectedCell.getId());
			if (cv != null) {
				cv.setSelected(true);
			}
			selectedCellViews.add(cv);
			view.repaint();
		}
	}
	
	
	
}
