package visualiser.viewpanel.view;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.Path2D;
import java.awt.geom.Point2D;
import java.util.List;

import rootgui.model.objects.*;
import rootgui.model.recordstore.SpeciesRecord;

/*
 * Representation of a cell 
 */
public class CellView {
	/*
	 * Background saturation value
	 */
	private double saturation = 0;

	/*
	 * Reference to the Cell object
	 */
	private Cell cell;
	/*
	 * Specifies if the cell is selected
	 */
	private boolean selected;
	/*
	 * Centre of the cell
	 */
	private Point centroid;

	/*
	 * Path2D object containing the coordinates of the cell's contour
	 * (perimeter)
	 */
	private Path2D contour;

	/*
	 * 
	 */
	private Path2D[] frame;

	/*
	 * 
	 */
	private double ratio = 0.2;

	public final static BasicStroke stroke = new BasicStroke(1.5f);
	public final static BasicStroke thinStroke = new BasicStroke(2.0f);
	final static float dash1[] = { 10.0f };
	public final static BasicStroke dashed = new BasicStroke(3.0f,
			BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10.0f, dash1, 0.0f);

	public CellView(Cell cell) {

		contour = new Path2D.Double();
		this.cell = cell;
		createPolygonWithPoints();
		createFrame();
	}

	/*
	 * 
	 */
	private void createPolygonWithPoints() {
		centroid = new Point(0, 0);
		int[] xpoints = cell.getXpoints();
		int[] ypoints = cell.getYpoints();
		int size = cell.sizeOfPoints();
		for (int i = 0; i < size; i++) {
			int x = xpoints[i];
			int y = ypoints[i];
			if (i == 0) {
				contour.moveTo(x, y);
				centroid.setX(x);
				centroid.setY(y);
			} else {
				contour.lineTo(x, y);
				centroid.incX(x);
				centroid.incY(y);
			}
		}
		centroid.setX(centroid.getX() / xpoints.length);
		centroid.setY(centroid.getY() / xpoints.length);
	}

	private void createFrame() {
		List<Wall> walls = cell.getWalls();
		int size = walls.size();
		frame = new Path2D.Double[size];
		int j = 0;
		for (Wall w : walls) {
			frame[j] = new Path2D.Double();
			List<Point> points = w.getPoints();
			int i = 0;
			for (Point p : points) {
				int x = p.getX();
				int y = p.getY();

				double ratioX = 0;
				double ratioY = 0;
				double diffX = centroid.getX() - x;
				double diffY = centroid.getY() - y;
				if (Math.abs(diffX) > 26 ) 
					ratioX = 0.05;
				else
					ratioX = 0.2;
				if( Math.abs(diffY) > 26)
					ratioY = 0.05;
				else
					ratioY = 0.2;
				
				double a = x + diffX * ratioX;
				double b = y + diffY * ratioY;
				if (i == 0) {
					frame[j].moveTo(a, b);
				} else {
					frame[j].lineTo(a, b);
				}
				i++;
			}
			j++;
		}
	}

	public Point getCentroid() {
		return centroid;
	}

	public Path2D getPolygon() {
		return contour;
	}

	public boolean contains(Point2D p) {
		return contour.contains(p);
	}

	/*
	 * Permanently transforms the polygon
	 */
	public void transform(AffineTransform at) {
		contour.transform(at);
		for (Path2D pin : frame)
			pin.transform(at);
	}

	public Path2D[] getFrame() {
		return frame;
	}

	public double setSaturation(double minCon, double maxCon, double cellCon) {
		if (maxCon == minCon)
			saturation = 0;
		else
			saturation = (cellCon - minCon) / (maxCon - minCon);
		return saturation;
	}

	public double getSaturation() {
		return saturation;
	}

	public boolean isSelected() {
		return selected;
	}

	public void setSelected(boolean selected) {
		this.selected = selected;
	}

	public Cell getCell() {
		return cell;
	}

	public int getCellID() {
		return cell.getId();
	}
}
