package visualiser.viewpanel.view.cellrendering;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.awt.geom.Path2D;

import java.util.Collection;

import rootgui.model.objects.Species;
import rootgui.model.recordstore.Record;
import rootgui.model.recordstore.SpeciesRecord;

import visualiser.Figure;
import visualiser.viewpanel.view.CellDrawingPanel;
import visualiser.viewpanel.view.CellView;
import visualiser.viewpanel.view.ColourHelper;

/*
 * An implementation of AbsCellRenderer where the cell is drawn only in terms of its contour and a background colour
 */
public class BackgroundCellRenderer extends AbsCellRenderer {
	private double maxRange = 0;
	private double minRange = 0;
	private boolean dynamicMinMaxCalculation = true;
	private double percentile = 0.95;
	@Override
	public void setFigure(Figure figure, int figureIndex, Record r) {
		super.setFigure(figure, figureIndex, r);
		maxRange = figure.getMaxRange(figureIndex);
		minRange = figure.getMinRange(figureIndex);
//		dynamicMinMaxCalculation = maxRange <= 0;
		if (dynamicMinMaxCalculation && sr != null) {
			double newMaxRange = sr.getMaxValue(percentile);
//			if (newMaxRange > maxRange)
				maxRange = newMaxRange;
		}
		if (sr==null){
			maxRange =1;
			minRange =0;
		}
	}

	public void setDynamicMinMaxCalculation(boolean b) {
		dynamicMinMaxCalculation = b;
	}
	
	public double getMinRange(){
		return minRange;
	}

	public double getMaxRange(){
		return maxRange;
	}
	
	public void setMaxRange(double maxValue){
		maxRange = maxValue;
	}
	
	@Override
	public void newIteration(Record r) {
		try {
			Species species = figure.getSpecies(figureIndex);
			if (r != null && species != null) {
				sr = r.getData(species);
				if (sr != null && dynamicMinMaxCalculation) {
					double newMaxRange = sr.getMaxValue(percentile);
					if (newMaxRange > maxRange)
						maxRange = newMaxRange;
				}
			} else {
				maxRange = 0;
				if (species == null) System.err.println("BackgroundCellRenderer.newIteration, species null");
			}
		} catch (Exception e) {
			 e.printStackTrace();
		}
		// if (sr == null)
		// System.err
		// .println("background cell render new interation is null!");
	}

	@Override
	public void drawCells(Collection<CellView> cvs, Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		// saving the state
		Color c = g2.getColor();
		Stroke s = g2.getStroke();

		int colourBarType = cdp.getColourBarType();
		for (CellView cv : cvs) {
			double saturation = 0;
			try {
				double v = sr.get(cv.getCellID()); // get species value
				saturation = cv.setSaturation(minRange, maxRange, v);
				if (saturation < 0)
					saturation = 0;
				else if (saturation > 1)
					saturation = 1;
			} catch (Exception e) {
			}

			Color surfaceColour;
			if (colourBarType == 0) {
				surfaceColour = ColourHelper.numberToColorPercentage(saturation);
			} else {
				int RGB = Color.HSBtoRGB(0.31F, (float) saturation, 1F);
				surfaceColour = new Color(RGB);
			}

			// drawing
			g2.setColor(surfaceColour);
			Path2D polygon = cv.getPolygon();
			g2.fill(polygon);
		}
		// restoring the state
		g2.setColor(c);
		g2.setStroke(s);
	}
}
