package visualiser.viewpanel.view.tabs;

import javax.swing.*;
import javax.swing.plaf.basic.BasicButtonUI;

import rootgui.view.SpeciesSelectionDialog;

import visualiser.Figure;
import visualiser.IVisualiser;
import visualiser.Visualiser;

import java.awt.*;
import java.awt.event.*;

/**
 * Component to be used as tabComponent; Contains a JLabel to show the text and
 * a JButton to close the tab it belongs to
 */
public class ButtonTabComponent extends JPanel {
	private final JTabbedPane pane;
	private IVisualiser vis;

	// public ButtonTabComponent(final JTabbedPane pane, IVisualiser vis) {
	// this(pane, "cross.gif", "Close this", vis);
	// }

	/**
	 * Constructor
	 * 
	 * @param pane
	 * @param icon
	 * @param toolTip
	 * @param m
	 */
	public ButtonTabComponent(final JTabbedPane pane, ImageIcon icon,
			String toolTip, IVisualiser vis) {
		super(new FlowLayout(FlowLayout.LEFT, 0, 0));
		this.vis = vis;

		if (pane == null) {
			throw new NullPointerException("TabbedPane is null");
		}
		this.pane = pane;
		setOpaque(false);

		// make JLabel read titles from JTabbedPane
		JLabel label = new JLabel() {
			public String getText() {
				int i = pane.indexOfTabComponent(ButtonTabComponent.this);
				if (i != -1) {
					return pane.getTitleAt(i);
				}
				return null;
			}
		};

		add(label);
		// add more space between the label and the button
		label.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
		// tab button
		JButton button = new TabButton(icon, toolTip, vis);
		add(button);
		// add more space to the top of the component
		setBorder(BorderFactory.createEmptyBorder(2, 0, 0, 0));
	}

	/**
	 * create a tab button
	 * 
	 * @author Lei Yu
	 * 
	 */
	private class TabButton extends JButton implements ActionListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		IVisualiser vis;

		public TabButton(ImageIcon closeIcon, String toolTip, IVisualiser vis) {
			this.vis = vis;
			int size = 17;
			setPreferredSize(new Dimension(size, size));

			setToolTipText(toolTip);
			setUI(new BasicButtonUI()); // Make the button looks the same for
										// all Laf's
			setContentAreaFilled(false); // Make it transparent
			setFocusable(false); // No need to be focusable
			setBorder(BorderFactory.createEtchedBorder());
			setBorderPainted(false);
			addMouseListener(buttonMouseListener); // we use the same listener
													// for all buttons
			setRolloverEnabled(true); // Making nice rollover effect
			setIcon(closeIcon);
			addActionListener(this); // Close the proper tab by clicking the
										// button
		}

		public void actionPerformed(ActionEvent e) {
			/* if(!model.isJTabInUse()){ */

			int m = pane.indexOfTabComponent(ButtonTabComponent.this);
			if (m == pane.getTabCount() - 1) { // add tab button

				// SpeciesSelectionDialog.showDialog(this, null, "Species: ",
				// "Select species", vis.getTabHistDataProvider()
				// .getLoadedSpeciesArray(vis.getSelectedTabIndex()));
				// if (SpeciesSelectionDialog.getButtonAction() ==
				// SpeciesSelectionDialog.OK
				// && SpeciesSelectionDialog.getSpecies().length > 0) {
				// Figure f = new Figure(SpeciesSelectionDialog.getSpecies(),
				// SpeciesSelectionDialog.getMinRanges(),
				// SpeciesSelectionDialog.getMaxRanges(),
				// SpeciesSelectionDialog.getSelectedRows());
				// f.setNoColumns(SpeciesSelectionDialog.getNoColumns());
				vis.addVisualiserTab(null);
				int i = pane.getTabCount() - 3;
				pane.remove(i);
				// vis.setSelectedTab(i);
				// vi.setSelectedIndex(i);
				// }
			} else {				
				if (m != -1 && pane.getTabCount() > 2) {  // close tab
					vis.removeTabByIndex(m);
				}
				// if (pane.getTabCount() != 2) {
				// vis.setSelectedTab(pane.getTabCount() - 2);
				// }
			}
			// }
		}

		// we don't want to update UI for this button
		public void updateUI() {
		}
	}

	private final static MouseListener buttonMouseListener = new MouseAdapter() {
		public void mouseEntered(MouseEvent e) {
			Component component = e.getComponent();
			if (component instanceof AbstractButton) {
				AbstractButton button = (AbstractButton) component;
				button.setBorderPainted(true);
			}
		}

		public void mouseExited(MouseEvent e) {
			Component component = e.getComponent();
			if (component instanceof AbstractButton) {
				AbstractButton button = (AbstractButton) component;
				button.setBorderPainted(false);
			}
		}
	};
}
